#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
get_ipython().run_line_magic('matplotlib', 'inline')
import seaborn as sns
import os
import re
import time
from sklearn.metrics import confusion_matrix
from sklearn.metrics.classification import log_loss
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.feature_extraction.text import CountVectorizer
from collections import Counter
from scipy.sparse import hstack
from scipy.sparse import csr_matrix
import warnings
warnings.filterwarnings("ignore")
from datetime import datetime
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier
from sklearn.calibration import CalibratedClassifierCV
from xgboost import XGBClassifier
import xgboost as xgb
from sklearn.model_selection import RandomizedSearchCV
from sklearn.preprocessing import LabelEncoder
from sklearn.preprocessing import StandardScaler
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.model_selection import GridSearchCV
from sklearn.calibration import CalibratedClassifierCV
from tqdm import tqdm
import joblib
from sklearn.externals import joblib as jobl
from joblib import dump
from keras.models import Sequential
from keras.layers import Dense, Dropout, Activation, BatchNormalization,Input,PReLU
from keras.utils import np_utils
#from keras.optimizers import adam_v2
from keras.models import Model
#from keras.optimizers import Adagrad
import datetime
from keras.models import load_model
from IPython.display import Image
from keras.callbacks import EarlyStopping,TensorBoard


# In[ ]:


print("# Loading Data")
ageGenderTrain = pd.read_csv('train_age.csv', dtype={'device_id':np.str})
newtrain = pd.read_csv('predictions_lgb_040.csv', dtype={'device_id':np.str})
ageGenderTest = pd.read_csv('test_age.csv', dtype={'device_id':np.str})
phoneBrand = pd.read_csv('phone_device_model.csv', dtype={'device_id':np.str})

eventsData = pd.read_csv('events.csv', parse_dates=['timestamp'], dtype={'device_id':np.str, 'event_id':np.str})
appEventsData = pd.read_csv('app_events.csv', dtype={'app_id':np.str, 'event_id':np.str})
labelCat = pd.read_csv('app_data.csv', dtype={'category':np.str,'app_id':np.str})
print("# Data Loaded")


# In[ ]:


#ageGenderTrain = ageGenderTrain[ageGenderTrain['value']<75]


# In[ ]:


'''
combine=[ageGenderTrain]

for dataset in combine:
    dataset.loc[dataset['value']<=14,'group']='13'
    dataset.loc[(dataset['value'] > 14) & (dataset['value'] <= 16), 'group'] = '16'

sns.countplot('value', hue='value', data=ageGenderTrain)
'''


# In[ ]:


ageGenderTrain.to_csv('train_data_all.csv',index=False)


# In[ ]:


phoneBrand.drop_duplicates(subset='device_id', keep='first',inplace=True)


# In[ ]:


#fact_list=labelCat['category'].value_counts()[0:85]
#labelCat['category']=labelCat['category'].apply(lambda x:'else' if x not in fact_list else x)


# In[ ]:


print("Train Size:", ageGenderTrain.shape)
print("Test Size:", ageGenderTest.shape)
trainrec = ageGenderTrain.shape[0]
testrec = ageGenderTest.shape[0]
ageGenderTrain.head()


# In[ ]:


trainEventData = ageGenderTrain.loc[ageGenderTrain.device_id.isin(eventsData.device_id)]
testEventData = ageGenderTest.loc[ageGenderTest.device_id.isin(eventsData.device_id)]


# In[ ]:


uBrands = phoneBrand.phone_brand.nunique()
uModels = phoneBrand.device_model.nunique()
uEvents = eventsData.event_id.nunique()
uCats = labelCat.category.nunique()
print(uBrands, uModels, uEvents, uCats)


# In[ ]:


# REF - https://www.kaggle.com/shahnawazakhtar/using-features-from-label-categories-csv
def vectorize(train, test, columns, fillMissing):
    data = pd.concat((train, test), axis=0, ignore_index=True)
    data = data[columns].astype(np.str).apply(lambda x: " ".join(s for s in x), axis=1).fillna(fillMissing)
    print(data.shape)
    split_len = len(train)
    # TF-IDF Feature
    print("TfidfVectorizer for", columns)
    vectorizer = TfidfVectorizer(min_df=1)
    data = vectorizer.fit_transform(data)

    train = data[:split_len, :]
    test = data[split_len:, :]
    return train, test


# In[ ]:


def vectorizeOHE(train, test, columns, fillMissing):
    data = pd.concat((train, test), axis=0, ignore_index=True)
    data = data[columns].astype(np.str).apply(lambda x: " ".join(s for s in x), axis=1).fillna(fillMissing)
    print(data.shape)
    split_len = len(train)
    # OHE Feature
    print("One Hot Encoding for", columns)
    vectorizer = CountVectorizer(min_df=1)
    data = vectorizer.fit_transform(data)
    train = data[:split_len, :]
    test = data[split_len:, :]
    return train, test


# In[ ]:


print("--Handling Labels Categories")
#---------------------------------------------------------------------------
labelCat = labelCat.fillna('unknown')
labelCat.drop_duplicates(keep='first', inplace=True)
#---------------------------------------------------------------------------
labelCat["category"] = labelCat["category"].apply(lambda x: " ".join(str(x).replace("-"," ").replace("/"," ").replace("("," ").replace(")"," ").split()))
labelCat.category = labelCat.category.apply(lambda x: "Cat:"+str(x))
categories = labelCat.groupby('app_id')['category'].apply(lambda x: " ".join(s for s in x))
labelCat = labelCat.drop('category', axis=1)


# In[ ]:


eventsData['timestamp2'] = pd.to_datetime(eventsData['timestamp'])


# In[ ]:


eventsData['hour'] = eventsData['timestamp2'].dt.hour


# In[ ]:


time_large = eventsData.groupby('device_id')['hour'].apply(lambda x: max(x))


# In[ ]:


eventsData = eventsData.drop(['timestamp2'], axis=1)


# In[ ]:


time_small = eventsData.groupby('device_id')['hour'].apply(lambda x: min(x))


# In[ ]:


from collections import Counter
time_most = eventsData.groupby('device_id')['hour'].apply(lambda x: Counter(x).most_common(1)[0][0])


# In[ ]:


trainEventData["time_large"] = trainEventData["device_id"].map(time_large)
testEventData["time_large"] = testEventData["device_id"].map(time_large)


# In[ ]:


trainEventData["time_small"] = trainEventData["device_id"].map(time_small)
testEventData["time_small"] = testEventData["device_id"].map(time_small)


# In[ ]:


trainEventData["time_most"] = trainEventData["device_id"].map(time_most)
testEventData["time_most"] = testEventData["device_id"].map(time_most)


# In[ ]:


trainEventData.head()


# In[ ]:


#---------------------------------------------------------------------------
print("----APP-EVENTS")
appEventsData["category"] = appEventsData["app_id"].map(categories)
categories = appEventsData.groupby("event_id")["category"].apply(lambda x: " ".join(str(s) for s in x))

appEventsData = appEventsData.drop('category', axis=1)
#---------------------------------------------------------------------------


# In[ ]:


#---------------------------------------------------------------------------
print("----EVENTS")
eventsData["category"] = eventsData["event_id"].map(categories)
categories = eventsData.groupby("device_id")["category"].apply(lambda x: " ".join(str(s) for s in x))

eventsData = eventsData.drop('category', axis=1)
#---------------------------------------------------------------------------


# In[ ]:


#---------------------------------------------------------------------------
print("----TRAIN-TEST Merge")
#ageGenderTrain["category"] = ageGenderTrain["device_id"].map(categories)
#ageGenderTest["category"] = ageGenderTest["device_id"].map(categories)
trainEventData["category"] = trainEventData["device_id"].map(categories)
testEventData["category"] = testEventData["device_id"].map(categories)

#---------------------------------------------------------------------------
del categories
#---------------------------------------------------------------------------
trainEventData.head()


# In[ ]:


trainEventData['catCnt'] = trainEventData.category.apply(lambda x: list(set(str(x).split('Cat:'))))
#trainEventData.category = trainEventData.catCnt.astype(str)
trainEventData.catCnt = trainEventData.catCnt.str.len()
trainEventData.catCnt = trainEventData.catCnt.fillna(0)

testEventData['catCnt'] = testEventData.category.apply(lambda x: list(set(str(x).split('Cat:'))))
#testEventData.category = testEventData.catCnt.astype(str)
testEventData.catCnt = testEventData.catCnt.str.len()
testEventData.catCnt = testEventData.catCnt.fillna(0)

trainEventData.head()


# In[ ]:


vectorizer=StandardScaler()
vectorizer.fit(trainEventData['catCnt'].values.reshape(-1,1))

Xtr_lCatCnt = vectorizer.transform(trainEventData['catCnt'].values.reshape(-1,1))
Xte_lCatCnt = vectorizer.transform(testEventData['catCnt'].values.reshape(-1,1))

print("Train Category-Count Shape: ",Xtr_lCatCnt.shape)
print("Test Category-Count Shape: ",Xte_lCatCnt.shape)


# In[ ]:


vectorizer=StandardScaler()
vectorizer.fit(trainEventData['time_large'].values.reshape(-1,1))

Xtr_time_large = vectorizer.transform(trainEventData['time_large'].values.reshape(-1,1))
Xte_ltime_large = vectorizer.transform(testEventData['time_large'].values.reshape(-1,1))

print("Train Category-Count Shape: ",Xtr_time_large.shape)
print("Test Category-Count Shape: ",Xte_ltime_large.shape)


# In[ ]:


vectorizer=StandardScaler()
vectorizer.fit(trainEventData['time_most'].values.reshape(-1,1))

Xtr_time_most = vectorizer.transform(trainEventData['time_most'].values.reshape(-1,1))
Xte_time_most = vectorizer.transform(testEventData['time_most'].values.reshape(-1,1))

print("Train Category-Count Shape: ",Xtr_time_most.shape)
print("Test Category-Count Shape: ",Xte_time_most.shape)


# In[ ]:


vectorizer=StandardScaler()
vectorizer.fit(trainEventData['time_small'].values.reshape(-1,1))

Xtr_time_small = vectorizer.transform(trainEventData['time_small'].values.reshape(-1,1))
Xte_time_small = vectorizer.transform(testEventData['time_small'].values.reshape(-1,1))

print("Train Category-Count Shape: ",Xtr_time_small.shape)
print("Test Category-Count Shape: ",Xte_time_small.shape)


# In[ ]:


Xtr_catCntProp = ((trainEventData.catCnt)/uCats).values.reshape(-1,1)
Xte_catCntProp = ((testEventData.catCnt)/uCats).values.reshape(-1,1)

print("Train Category-Count Proportion Shape: ",Xtr_catCntProp.shape)
print("Test Category-Count Proportion Shape: ",Xte_catCntProp.shape)


# In[ ]:


Xtr_lCatEv, Xte_lCatEv = vectorize(trainEventData, testEventData, ["category"], "missing")
print(Xtr_lCatEv.shape)
print(Xte_lCatEv.shape)


# In[ ]:


trainEventData = trainEventData.drop(['category','catCnt'], axis=1)
testEventData = testEventData.drop(['category','catCnt'], axis=1)


# In[ ]:


print("--Handling APP IDs")
#---------------------------------------------------------------------------
labelap = LabelEncoder().fit(labelCat.app_id)
labelCat.app_id = labelap.transform(labelCat.app_id)
appCount = len(labelap.classes_)
print('Total unique apps: ', appCount)

appEventsData.app_id = labelap.transform(appEventsData.app_id)
#---------------------------------------------------------------------------
print("----APP-EVENTS")
appEventsData["appID"] = appEventsData.app_id.astype(str).apply(lambda x: "APPID:"+str(x))
appIDs = appEventsData.groupby("event_id")["appID"].apply(lambda x: " ".join(str(s) for s in x))

appEventsData = appEventsData.drop('appID', axis=1)
#---------------------------------------------------------------------------
print("----EVENTS")
eventsData["appID"] = eventsData["event_id"].map(appIDs)
appIDs = eventsData.groupby("device_id")["appID"].apply(lambda x: " ".join(str(s) for s in x))

eventsData = eventsData.drop('appID', axis=1)
#---------------------------------------------------------------------------
print("----TRAIN-TEST Merge")
#ageGenderTrain["appID"] = ageGenderTrain["device_id"].map(appIDs)
#ageGenderTest["appID"] = ageGenderTest["device_id"].map(appIDs)
trainEventData["appID"] = trainEventData["device_id"].map(appIDs)
testEventData["appID"] = testEventData["device_id"].map(appIDs)

#---------------------------------------------------------------------------
del appIDs
#---------------------------------------------------------------------------
trainEventData.head()


# In[ ]:


trainEventData['appIDCnt'] = trainEventData.appID.str.replace('APPID:','')
trainEventData.appIDCnt = trainEventData.appIDCnt.str.replace(' ',', ')
testEventData['appIDCnt'] = testEventData.appID.str.replace('APPID:','')
testEventData.appIDCnt = testEventData.appIDCnt.str.replace(' ',', ')
trainEventData.head()


# In[ ]:


testEventData.head()


# In[ ]:


trainEventData.appIDCnt = trainEventData.appIDCnt.apply(lambda x: list(set(str(x).split(', '))))
#trainEventData.appID = trainEventData.appIDCnt.astype(str)
trainEventData.appIDCnt = trainEventData.appIDCnt.str.len()

testEventData.appIDCnt = testEventData.appIDCnt.apply(lambda x: list(set(str(x).split(', '))))
#testEventData.appID = testEventData.appIDCnt.astype(str)
testEventData.appIDCnt = testEventData.appIDCnt.str.len()

trainEventData.head()


# In[ ]:


Xtr_appIDEv, Xte_appIDEv = vectorize(trainEventData, testEventData, ["appID"], "missing")
print(Xtr_appIDEv.shape)
print(Xte_appIDEv.shape)


# In[ ]:


vectorizer=StandardScaler()
vectorizer.fit(trainEventData['appIDCnt'].values.reshape(-1,1))

Xtr_appIDCnt = vectorizer.transform(trainEventData['appIDCnt'].values.reshape(-1,1))
Xte_appIDCnt = vectorizer.transform(testEventData['appIDCnt'].values.reshape(-1,1))

print("Train APPID-Count Shape: ",Xtr_appIDCnt.shape)
print("Test APPID-Count Shape: ",Xte_appIDCnt.shape)


# In[ ]:


Xtr_appIDCntProp = ((trainEventData.appIDCnt)/uCats).values.reshape(-1,1)
Xte_appIDCntProp = ((testEventData.appIDCnt)/uCats).values.reshape(-1,1)
print("Train APP ID Proportion Shape: ",Xtr_appIDCntProp.shape)
print("Test APP ID Proportion Shape: ",Xte_appIDCntProp.shape)


# In[ ]:


from scipy.stats import entropy


# In[ ]:


# REF - https://www.kaggle.com/c/talkingdata-mobile-user-demographics/discussion/23424
trainEventData.appID = trainEventData.appID.apply(lambda x: entropy(list(Counter(str(x).split(', ')).values()), base=2))
testEventData.appID = testEventData.appID.apply(lambda x: entropy(list(Counter(str(x).split(', ')).values()), base=2))

print(trainEventData.shape)
print(testEventData.shape)
trainEventData.head()


# In[ ]:


print("--Handling IS_ACTIVE")
#---------------------------------------------------------------------------
print("----APP-EVENTS")
appEventsData["activeApp"] = appEventsData.is_active.astype(str).apply(lambda x: " ".join(str(s) for s in x))
activeApps = appEventsData.groupby("event_id")["is_active"].apply(lambda x: " ".join(str(s) for s in x))

appEventsData = appEventsData.drop('activeApp', axis=1)
#---------------------------------------------------------------------------
print("----EVENTS")
eventsData["activeApp"] = eventsData["event_id"].map(activeApps)
activeApps = eventsData.groupby("device_id")["activeApp"].apply(lambda x: " ".join(str(s) for s in x))

eventsData = eventsData.drop('activeApp', axis=1)
#---------------------------------------------------------------------------
print("----TRAIN-TEST Merge")
#ageGenderTrain["activeApp"] = ageGenderTrain["device_id"].map(activeApps)
#ageGenderTest["activeApp"] = ageGenderTest["device_id"].map(activeApps)

trainEventData["activeApp"] = trainEventData["device_id"].map(activeApps)
testEventData["activeApp"] = testEventData["device_id"].map(activeApps)

del activeApps
#---------------------------------------------------------------------------
trainEventData.head()


# In[ ]:


Xtr_activeAppEv, Xte_activeAppEv = vectorize(trainEventData, testEventData, ["activeApp"], "missing")
print(Xtr_activeAppEv.shape)
print(Xte_activeAppEv.shape)


# In[ ]:


trainEventData['actvAppCnt'] = trainEventData.activeApp.str.count('1')
testEventData['actvAppCnt'] = testEventData.activeApp.str.count('1')

vectorizer=StandardScaler()
vectorizer.fit(trainEventData['actvAppCnt'].values.reshape(-1,1))

Xtr_actvAppCnt = vectorizer.transform(trainEventData['actvAppCnt'].values.reshape(-1,1))
Xte_actvAppCnt = vectorizer.transform(testEventData['actvAppCnt'].values.reshape(-1,1))

print("Train Active APP Count Shape: ",Xtr_actvAppCnt.shape)
print("Test Active APP Count Shape: ",Xte_actvAppCnt.shape)


# In[ ]:


trainEventData['avg_actvAppCnt'] = trainEventData['actvAppCnt']/7
testEventData['avg_actvAppCnt'] = testEventData['actvAppCnt']/7
vectorizer=StandardScaler()
vectorizer.fit(trainEventData['avg_actvAppCnt'].values.reshape(-1,1))

Xtr_avg_actvAppCnt = vectorizer.transform(trainEventData['avg_actvAppCnt'].values.reshape(-1,1))
Xte_avg_actvAppCnt = vectorizer.transform(testEventData['avg_actvAppCnt'].values.reshape(-1,1))


# In[ ]:


trainEventData.activeApp = trainEventData.activeApp.str.replace(' ',', ')
testEventData.activeApp = testEventData.activeApp.str.replace(' ',', ')

# REF - https://www.kaggle.com/c/talkingdata-mobile-user-demographics/discussion/23424
trainEventData.activeApp = trainEventData.activeApp.apply(lambda x: entropy(list(Counter(str(x).split(', ')).values()), base=2))
testEventData.activeApp = testEventData.activeApp.apply(lambda x: entropy(list(Counter(str(x).split(', ')).values()), base=2))

print(trainEventData.shape)
print(testEventData.shape)
trainEventData.head()


# In[ ]:


Xtr_activeAppEntropy = trainEventData['activeApp'].values.reshape(-1,1)
Xte_activeAppEntropy = testEventData['activeApp'].values.reshape(-1,1)

print("Train Active APP Entropy Shape: ",Xtr_activeAppEntropy.shape)
print("Test Active APP Entropy Shape: ",Xte_activeAppEntropy.shape)


# In[ ]:


trainEventData = trainEventData.drop(['activeApp', 'actvAppCnt','avg_actvAppCnt'], axis=1)
testEventData = testEventData.drop(['activeApp', 'actvAppCnt','avg_actvAppCnt'], axis=1)


# In[ ]:


print('Handling Phone-Brand and Device-Model')
phoneBrand = phoneBrand.drop_duplicates('device_id', keep='first')
phoneBrand.device_model = phoneBrand.phone_brand.str.cat(" "+phoneBrand.device_model)
phoneBrand.phone_brand = phoneBrand.phone_brand.map(str.strip).map(str.lower)
phoneBrand.device_model = phoneBrand.device_model.map(str.strip).map(str.lower)
uModels = phoneBrand.device_model.nunique()

ageGenderTrain = ageGenderTrain.merge(phoneBrand, on='device_id', how='left')
ageGenderTest = ageGenderTest.merge(phoneBrand, on='device_id', how='left')
#---------------------------------------------
print("Train:", ageGenderTrain.shape)
print("Test:", ageGenderTest.shape)
ageGenderTrain.head()


# In[ ]:


#notrainEventData = ageGenderTrain.loc[~ageGenderTrain.device_id.isin(eventsData.device_id)]
#notestEventData = ageGenderTest.loc[~ageGenderTest.device_id.isin(eventsData.device_id)]
notrainEventData = ageGenderTrain
notestEventData = ageGenderTest
trainEventData = ageGenderTrain.loc[ageGenderTrain.device_id.isin(eventsData.device_id)]
testEventData = ageGenderTest.loc[ageGenderTest.device_id.isin(eventsData.device_id)]


# In[ ]:


print(notestEventData)


# In[ ]:


phbrData = pd.concat([ageGenderTrain, ageGenderTest], axis=0, ignore_index=True)
print("TfidfVectorizer for Phone Brand")
vectorizer = TfidfVectorizer(min_df=1).fit(phbrData.phone_brand)
Xtr_phbrev = vectorizer.transform(trainEventData.phone_brand)
Xte_phbrev = vectorizer.transform(testEventData.phone_brand)
print("=====================================")
print("After TfidfVectorizer for PHONE BRAND")
print(Xtr_phbrev.shape)
print(Xte_phbrev.shape)
print("=====================================")

Xtr_phbrnoev = vectorizer.transform(notrainEventData.phone_brand)
Xte_phbrnoev = vectorizer.transform(notestEventData.phone_brand)

print("=====================================")
print("After TfidfVectorizer for PHONE BRAND")
print(Xtr_phbrnoev.shape)
print(Xte_phbrnoev.shape)
print("=====================================")


# In[ ]:


demoData = pd.concat([ageGenderTrain, ageGenderTest], axis=0, ignore_index=True)
print("TfidfVectorizer for Device Model")
vectorizer = TfidfVectorizer(min_df=1).fit(demoData.device_model)
Xtr_demoev = vectorizer.transform(trainEventData.device_model)
Xte_demoev = vectorizer.transform(testEventData.device_model)

print("=====================================")
print("After TfidfVectorizer for DEVICE MODEL")
print(Xtr_demoev.shape)
print(Xte_demoev.shape)
print("=====================================")

Xtr_demonoev = vectorizer.transform(notrainEventData.device_model)
Xte_demonoev = vectorizer.transform(notestEventData.device_model)

print("=====================================")
print("After TfidfVectorizer for DEVICE MODEL")
print(Xtr_demonoev.shape)
print(Xte_demonoev.shape)
print("=====================================")


# In[ ]:


ageGenderTrain = ageGenderTrain.drop(['phone_brand','device_model'], axis=1)
ageGenderTest = ageGenderTest.drop(['phone_brand','device_model'], axis=1)

trainEventData = trainEventData.drop(['phone_brand','device_model'], axis=1)
testEventData = testEventData.drop(['phone_brand','device_model'], axis=1)

notrainEventData = notrainEventData.drop(['phone_brand','device_model'], axis=1)
notestEventData = notestEventData.drop(['phone_brand','device_model'], axis=1)

print(ageGenderTrain.shape)
print(ageGenderTest.shape)

ageGenderTrain.head()


# In[ ]:


ageGenderTrain['trainrow'] = np.arange(ageGenderTrain.shape[0])
ageGenderTest['testrow'] = np.arange(ageGenderTest.shape[0])
#---------------------------------------------
print(ageGenderTrain.shape)
print(ageGenderTest.shape)
ageGenderTrain.head()


# In[ ]:


#---------------------------------------------------------------------------
trEventData = ageGenderTrain.loc[ageGenderTrain.device_id.isin(eventsData.device_id)]
teEventData = ageGenderTest.loc[ageGenderTest.device_id.isin(eventsData.device_id)]

evtrainrec = trEventData.shape[0]
evtestrec = teEventData.shape[0]
trEventData['trainrow'] = np.arange(trEventData.shape[0])
teEventData['testrow'] = np.arange(teEventData.shape[0])


# In[ ]:


def findDayPart(x):
    if (x>21 and x <=23) or (x>=0 and x<=6):
        return 0 #NIGHT
    if x>6 and x<9:
        return 1 #MORNING
    if x>=9 and x<=18:
        return 2 #PEAKHOURS
    if x>18 and x<22:
        return 3 #EVENING
#---------------------------------------------------------------------------
eventsData['dayOfWeek'] = eventsData.timestamp.dt.dayofweek
eventsData['activityhour'] = eventsData.timestamp.dt.hour
eventsData['dayPart'] = eventsData['activityhour'].apply(findDayPart)
eventsData = eventsData.drop(['timestamp'], axis=1)
print(eventsData.shape)
eventsData.head()


# In[ ]:


trEventData = trEventData.merge(eventsData[['device_id','dayOfWeek']], on='device_id', how='left')
teEventData = teEventData.merge(eventsData[['device_id','dayOfWeek']], on='device_id', how='left')
#---------------------------------------------------------------------------

Xtr_dayOfWeekEv = csr_matrix((np.ones(trEventData.shape[0]), (trEventData.trainrow, trEventData.dayOfWeek)), shape=(evtrainrec,7))
Xte_dayOfWeekEv = csr_matrix((np.ones(teEventData.shape[0]), (teEventData.testrow, teEventData.dayOfWeek)), shape=(evtestrec,7))
#---------------------------------------------------------------------------
print(Xtr_dayOfWeekEv.shape)
print(Xte_dayOfWeekEv.shape)


# In[ ]:


# REF - https://www.kaggle.com/c/talkingdata-mobile-user-demographics/discussion/23424
dowEntropy = eventsData[['device_id', 'dayOfWeek']].groupby('device_id')['dayOfWeek'].apply(lambda x: ", ".join(str(s) for s in x))

trEventData['dowEntropy'] = trEventData.device_id.map(dowEntropy)
trEventData.dowEntropy = trEventData.dowEntropy.apply(lambda x: entropy(list(Counter(str(x).split(', ')).values()), base=2))
teEventData['dowEntropy'] = teEventData.device_id.map(dowEntropy)
teEventData.dowEntropy = teEventData.dowEntropy.apply(lambda x: entropy(list(Counter(str(x).split(', ')).values()), base=2))

trEventData = trEventData.drop_duplicates('device_id')
teEventData = teEventData.drop_duplicates('device_id')

print(trEventData.shape)
print(teEventData.shape)
trEventData.head()


# In[ ]:


Xtr_dowEntropy = trEventData['dowEntropy'].values.reshape(-1,1)
Xte_dowEntropy = teEventData['dowEntropy'].values.reshape(-1,1)

print("Train EVENT-Count Shape: ",Xtr_dowEntropy.shape)
print("Test EVENT-Count Shape: ",Xte_dowEntropy.shape)

trEventData = trEventData.drop(['dowEntropy'], axis=1)
teEventData = teEventData.drop(['dowEntropy'], axis=1)


# In[ ]:


trEventData = trEventData.drop(['dayOfWeek'], axis=1)
teEventData = teEventData.drop(['dayOfWeek'], axis=1)

trEventData = trEventData.drop_duplicates('device_id')
teEventData = teEventData.drop_duplicates('device_id')

trEventData = trEventData.merge(eventsData[['device_id','activityhour']], on='device_id', how='left')
teEventData = teEventData.merge(eventsData[['device_id','activityhour']], on='device_id', how='left')

Xtr_activityhourEv = csr_matrix((np.ones(trEventData.shape[0]), (trEventData.trainrow, trEventData.activityhour)), shape=(evtrainrec,24))
Xte_activityhourEv = csr_matrix((np.ones(teEventData.shape[0]), (teEventData.testrow, teEventData.activityhour)), shape=(evtestrec,24))
#---------------------------------------------------------------------------
print(Xtr_activityhourEv.shape)
print(Xte_activityhourEv.shape)


# In[ ]:


hourEntropy = eventsData[['device_id', 'activityhour']].groupby('device_id')['activityhour'].apply(lambda x: ", ".join(str(s) for s in x))

trEventData['hourEntropy'] = trEventData.device_id.map(hourEntropy)
trEventData.hourEntropy = trEventData.hourEntropy.apply(lambda x: entropy(list(Counter(str(x).split(', ')).values()), base=2))
teEventData['hourEntropy'] = teEventData.device_id.map(hourEntropy)
teEventData.hourEntropy = teEventData.hourEntropy.apply(lambda x: entropy(list(Counter(str(x).split(', ')).values()), base=2))

trEventData = trEventData.drop_duplicates('device_id')
teEventData = teEventData.drop_duplicates('device_id')

print(trEventData.shape)
print(teEventData.shape)
trEventData.head()


# In[ ]:


Xtr_hourEntropy = trEventData['hourEntropy'].values.reshape(-1,1)
Xte_hourEntropy = teEventData['hourEntropy'].values.reshape(-1,1)

print("Train EVENT-Count Shape: ",Xtr_hourEntropy.shape)
print("Test EVENT-Count Shape: ",Xte_hourEntropy.shape)

trEventData = trEventData.drop(['hourEntropy'], axis=1)
teEventData = teEventData.drop(['hourEntropy'], axis=1)


# In[ ]:


trEventData = trEventData.drop(['activityhour'], axis=1)
teEventData = teEventData.drop(['activityhour'], axis=1)
trEventData = trEventData.drop_duplicates('device_id')
teEventData = teEventData.drop_duplicates('device_id')


trEventData = trEventData.merge(eventsData[['device_id','dayPart']], on='device_id', how='left')
teEventData = teEventData.merge(eventsData[['device_id','dayPart']], on='device_id', how='left')

Xtr_dayPartEv = csr_matrix((np.ones(trEventData.shape[0]), (trEventData.trainrow, trEventData.dayPart)), shape=(evtrainrec,4))
Xte_dayPartEv = csr_matrix((np.ones(teEventData.shape[0]), (teEventData.testrow, teEventData.dayPart)), shape=(evtestrec,4))
#---------------------------------------------------------------------------
print(Xtr_dayPartEv.shape)
print(Xte_dayPartEv.shape)


# In[ ]:


# REF - https://www.kaggle.com/c/talkingdata-mobile-user-demographics/discussion/23424
dayPartEntropy = eventsData[['device_id', 'dayPart']].groupby('device_id')['dayPart'].apply(lambda x: ", ".join(str(s) for s in x))

trEventData['dayPartEntropy'] = trEventData.device_id.map(dayPartEntropy)
trEventData.dayPartEntropy = trEventData.dayPartEntropy.apply(lambda x: entropy(list(Counter(str(x).split(', ')).values()), base=2))
teEventData['dayPartEntropy'] = teEventData.device_id.map(dayPartEntropy)
teEventData.dayPartEntropy = teEventData.dayPartEntropy.apply(lambda x: entropy(list(Counter(str(x).split(', ')).values()), base=2))

trEventData = trEventData.drop_duplicates('device_id')
teEventData = teEventData.drop_duplicates('device_id')

print(trEventData.shape)
print(teEventData.shape)
trEventData.head()


# In[ ]:


Xtr_dayPartEntropy = trEventData['dayPartEntropy'].values.reshape(-1,1)
Xte_dayPartEntropy = teEventData['dayPartEntropy'].values.reshape(-1,1)

print("Train EVENT-Count Shape: ",Xtr_dayPartEntropy.shape)
print("Test EVENT-Count Shape: ",Xte_dayPartEntropy.shape)

trEventData = trEventData.drop(['dayPartEntropy'], axis=1)
teEventData = teEventData.drop(['dayPartEntropy'], axis=1)


# In[ ]:


trEventData = trEventData.drop(['dayPart'], axis=1)
teEventData = teEventData.drop(['dayPart'], axis=1)

trEventData = trEventData.drop_duplicates('device_id')
teEventData = teEventData.drop_duplicates('device_id')


# In[ ]:


# REF - https://www.kaggle.com/zfturbo/xgboost-simple-starter
eventsData['evCount'] = eventsData.groupby(['device_id'])['event_id'].transform('count')
deviceEvents = eventsData[['device_id', 'evCount']].drop_duplicates('device_id', keep='first')
eventsData = eventsData.drop(['evCount'], axis=1)

trEventData = trEventData.merge(deviceEvents, on='device_id', how='left')
trEventData.evCount = trEventData.evCount.fillna(0)
teEventData = teEventData.merge(deviceEvents, on='device_id', how='left')
teEventData.evCount = teEventData.evCount.fillna(0)

print(trEventData.shape)
print(teEventData.shape)
trEventData.head()


# In[ ]:


vectorizer=StandardScaler()
vectorizer.fit(trEventData['evCount'].values.reshape(-1,1))

Xtr_evCount = vectorizer.transform(trEventData['evCount'].values.reshape(-1,1))
Xte_evCount = vectorizer.transform(teEventData['evCount'].values.reshape(-1,1))

print("Train EVENT-Count Shape: ",Xtr_evCount.shape)
print("Test EVENT-Count Shape: ",Xte_evCount.shape)


# In[ ]:


y = ageGenderTrain['value']
#ohe = LabelEncoder().fit(y.astype(str))
#y = ohe.transform(y.astype(str))
print(y)


# In[ ]:


ynoev = notrainEventData['value']
#ynoev = ohe.transform(ynoev.astype(str))
print(ynoev)


# In[ ]:


yev = trainEventData['value']
#yev = ohe.transform(yev.astype(str))
#print(yev)


# In[ ]:


from scipy.sparse import hstack
Xnoev_tr = hstack((Xtr_phbrnoev, Xtr_demonoev)).tocsr()
Xnoev_te = hstack((Xte_phbrnoev, Xte_demonoev)).tocsr()

print("====================================")
print("After Merging all the Sparse Matrices")
print(Xnoev_tr.shape)
print(Xnoev_te.shape)
print("====================================")


# In[ ]:


from scipy.sparse import hstack
Xev_tr = hstack((Xtr_time_large,Xtr_time_most,Xtr_time_most,Xtr_lCatEv, Xtr_lCatCnt, Xtr_catCntProp, Xtr_appIDEv, Xtr_appIDCnt, Xtr_appIDCntProp, Xtr_appIDEntropy, Xtr_activeAppEv, Xtr_actvAppCnt, Xtr_activeAppEntropy, Xtr_phbrev, Xtr_demoev, Xtr_dayOfWeekEv, Xtr_dowEntropy, Xtr_activityhourEv, Xtr_hourEntropy, Xtr_dayPartEv, Xtr_dayPartEntropy, Xtr_evCount)).tocsr()
Xev_te = hstack((Xte_ltime_large,Xte_time_most,Xte_time_most,Xte_lCatEv, Xte_lCatCnt, Xte_catCntProp, Xte_appIDEv, Xte_appIDCnt, Xte_appIDCntProp, Xte_appIDEntropy, Xte_activeAppEv, Xte_actvAppCnt, Xte_activeAppEntropy, Xte_phbrev, Xte_demoev, Xte_dayOfWeekEv, Xte_dowEntropy, Xte_activityhourEv, Xte_hourEntropy, Xte_dayPartEv, Xte_dayPartEntropy, Xte_evCount)).tocsr()

print("====================================")
print("After Merging all the Sparse Matrices")
print(Xev_tr.shape)
print(Xev_te.shape)
print("====================================")


# In[ ]:


print(Xtr_lCatEv) #0.017581295408285763


# In[ ]:


def save_sparse_matrix(filename, xmtr):
    np.savez(filename,data = xmtr.data ,indices= xmtr.indices,
             indptr =xmtr.indptr, shape=xmtr.shape )
    
#Loads a sparse matrix
def load_sparse_matrix(filename):
    tmp = np.load(filename)
    return csr_matrix((tmp['data'], tmp['indices'], tmp['indptr']), shape= tmp['shape'])


# In[ ]:


save_sparse_matrix('sparse/Xev_tr.npz',Xev_tr.tocsr())
save_sparse_matrix('sparse/Xev_te.npz',Xev_te.tocsr())




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import pandas as pd
import os
import gc
import lightgbm as lgb
import xgboost as xgb
from catboost import CatBoostRegressor
from sklearn.linear_model import SGDRegressor, LinearRegression, Ridge
from sklearn.preprocessing import MinMaxScaler
from gensim.models import Word2Vec
import math
import numpy as np
from tqdm import tqdm
from sklearn.model_selection import StratifiedKFold, KFold
from sklearn.metrics import accuracy_score, f1_score, roc_auc_score, log_loss
import matplotlib.pyplot as plt
import time
import warnings
warnings.filterwarnings('ignore')


# In[ ]:


train = pd.read_csv('train.csv')
test = pd.read_csv('test.csv')


# In[ ]:


train.columns


# In[ ]:


data = pd.concat([train, test], axis=0, ignore_index=True)


# ## 训练数据/测试数据准备

# In[ ]:


features = [f for f in data.columns if f not in ['是否流失','客户ID']]

train = data[data['是否流失'].notnull()].reset_index(drop=True)
test = data[data['是否流失'].isnull()].reset_index(drop=True)

x_train = train[features]
x_test = test[features]

y_train = train['是否流失']


# ## 模型训练
# - 直接构建了一个函数，可以调用三种树模型，方便快捷

# In[ ]:


def cv_model(clf, train_x, train_y, test_x, clf_name):
    folds = 5
    seed = 2022
    kf = KFold(n_splits=folds, shuffle=True, random_state=seed)

    train = np.zeros(train_x.shape[0])
    test = np.zeros(test_x.shape[0])

    cv_scores = []

    for i, (train_index, valid_index) in enumerate(kf.split(train_x, train_y)):
        print('************************************ {} ************************************'.format(str(i+1)))
        trn_x, trn_y, val_x, val_y = train_x.iloc[train_index], train_y[train_index], train_x.iloc[valid_index], train_y[valid_index]

        if clf_name == "lgb":
            train_matrix = clf.Dataset(trn_x, label=trn_y)
            valid_matrix = clf.Dataset(val_x, label=val_y)

            params = {
                'boosting_type': 'gbdt',
                'objective': 'binary',
                'metric': 'auc',
                'min_child_weight': 5,
                'num_leaves': 2 ** 5,
                'lambda_l2': 10,
                'feature_fraction': 0.7,
                'bagging_fraction': 0.7,
                'bagging_freq': 10,
                'learning_rate': 0.2,
                'seed': 2022,
                'n_jobs':-1
            }

            model = clf.train(params, train_matrix, 50000, valid_sets=[train_matrix, valid_matrix], 
                              categorical_feature=[], verbose_eval=3000, early_stopping_rounds=200)
            val_pred = model.predict(val_x, num_iteration=model.best_iteration)
            test_pred = model.predict(test_x, num_iteration=model.best_iteration)
            
            print(list(sorted(zip(features, model.feature_importance("gain")), key=lambda x: x[1], reverse=True))[:20])
                
        if clf_name == "xgb":
            train_matrix = clf.DMatrix(trn_x , label=trn_y)
            valid_matrix = clf.DMatrix(val_x , label=val_y)
            test_matrix = clf.DMatrix(test_x)
            
            params = {'booster': 'gbtree',
                      'objective': 'binary:logistic',
                      'eval_metric': 'auc',
                      'gamma': 1,
                      'min_child_weight': 1.5,
                      'max_depth': 5,
                      'lambda': 10,
                      'subsample': 0.7,
                      'colsample_bytree': 0.7,
                      'colsample_bylevel': 0.7,
                      'eta': 0.2,
                      'tree_method': 'exact',
                      'seed': 2020,
                      'nthread': 36,
                      "silent": True,
                      }
            
            watchlist = [(train_matrix, 'train'),(valid_matrix, 'eval')]
            
            model = clf.train(params, train_matrix, num_boost_round=50000, evals=watchlist, verbose_eval=3000, early_stopping_rounds=200)
            val_pred  = model.predict(valid_matrix, ntree_limit=model.best_ntree_limit)
            test_pred = model.predict(test_matrix , ntree_limit=model.best_ntree_limit)
                 
        if clf_name == "cat":
            params = {'learning_rate': 0.2, 'depth': 5, 'l2_leaf_reg': 10, 'bootstrap_type': 'Bernoulli',
                      'od_type': 'Iter', 'od_wait': 50, 'random_seed': 11, 'allow_writing_files': False}
            
            model = clf(iterations=20000, **params)
            model.fit(trn_x, trn_y, eval_set=(val_x, val_y),
                      cat_features=[], use_best_model=True, verbose=3000)
            
            val_pred  = model.predict(val_x)
            test_pred = model.predict(test_x)
            
        train[valid_index] = val_pred
        test = test_pred / kf.n_splits
        cv_scores.append(roc_auc_score(val_y, val_pred))
        
        print(cv_scores)
       
    print("%s_scotrainre_list:" % clf_name, cv_scores)
    print("%s_score_mean:" % clf_name, np.mean(cv_scores))
    print("%s_score_std:" % clf_name, np.std(cv_scores))
    return train, test


# In[ ]:


def lgb_model(x_train, y_train, x_test):
    lgb_train, lgb_test = cv_model(lgb, x_train, y_train, x_test, "lgb")
    return lgb_train, lgb_test

def xgb_model(x_train, y_train, x_test):
    xgb_train, xgb_test = cv_model(xgb, x_train, y_train, x_test, "xgb")
    return xgb_train, xgb_test

def cat_model(x_train, y_train, x_test):
    cat_train, cat_test = cv_model(CatBoostRegressor, x_train, y_train, x_test, "cat") 
    return cat_train, cat_test


# In[ ]:


lgb_train, lgb_test = lgb_model(x_train, y_train, x_test)


# In[ ]:


xgb_train, xgb_test = xgb_model(x_train, y_train, x_test)


# In[ ]:


cat_train, cat_test = cat_model(x_train, y_train, x_test)


# In[ ]:


test['是否流失'] = lgb_test


# In[ ]:


test[['客户ID','是否流失']].to_csv('test_sub.csv', index=False)


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import os
import math
import time
import random
import datetime
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import tensorflow as tf
import tensorflow.keras as K
from tensorflow.keras import Sequential, utils, regularizers, Model, Input
from tensorflow.keras.layers import Flatten, Dense, Conv1D, MaxPool1D, Dropout, AvgPool1D
from imblearn.over_sampling import SMOTE
from sklearn.model_selection import KFold
from sklearn.preprocessing import OneHotEncoder
import pickle
from scipy.sparse import csr_matrix, hstack
from collections import Counter
from scipy.stats import entropy
from statistics import mean
from keras import regularizers
import warnings
warnings.filterwarnings("ignore")
from sklearn.model_selection import train_test_split, RandomizedSearchCV
from sklearn.preprocessing import OneHotEncoder, LabelBinarizer, Normalizer, LabelEncoder, StandardScaler
from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer, HashingVectorizer
import os
from sklearn.neighbors import KNeighborsClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier
from xgboost import XGBClassifier
import xgboost as xgb
from lightgbm import LGBMClassifier
import lightgbm as lgb
import tensorflow as tf
from tensorflow.keras.layers import Input, Dense, Conv2D, MaxPool2D, Activation, Dropout, Flatten, PReLU, BatchNormalization
from tensorflow.keras.applications import VGG16
from tensorflow.keras.models import Model, Sequential
from tensorflow.keras.callbacks import Callback, ModelCheckpoint, TensorBoard, EarlyStopping, ReduceLROnPlateau
from tensorflow.keras.preprocessing.image import ImageDataGenerator
from tensorflow.keras.utils import plot_model
from tensorflow.keras.optimizers import Adagrad, Adam, Adadelta, Adamax
from keras.utils import np_utils
from sklearn.calibration import CalibratedClassifierCV
from sklearn.metrics import log_loss
from prettytable import PrettyTable

train = pd.read_csv('datasets/@huangzx23#ba5d051838cf583c27a68007e887d01c/train.csv')
test = pd.read_csv('datasets/@huangzx23#ba5d051838cf583c27a68007e887d01c/test.csv')
y_train = train['label']
x_train = train.drop(['label'], axis=1)
X_test = test
print(x_train.shape, y_train.shape)
print(X_test.shape)


# In[ ]:


import numpy as np
from joblib import Parallel, delayed
from scipy.interpolate import interp1d
from scipy.special import erf, erfinv
from sklearn.preprocessing import QuantileTransformer,PowerTransformer
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.utils.validation import FLOAT_DTYPES, check_array, check_is_fitted


# In[ ]:


#查看类别分布
train2 = pd.read_csv('submit_B.csv')

plt.hist(train['label'], orientation = 'vertical', histtype = 'bar', color = 'red')
plt.show() 

plt.hist(train2['label'], orientation = 'vertical', histtype = 'bar', color = 'red')
plt.show() 


# In[ ]:


# 使用 SMOTE 对数据进行上采样以解决类别不平衡问题
smote = SMOTE(random_state=2021, n_jobs=-1)
k_x_train, k_y_train = smote.fit_resample(x_train, y_train)  
print(f"after smote, k_x_train.shape: {k_x_train.shape}, k_y_train.shape: {k_y_train.shape}")
# 将训练集转换为适应 CNN 输入的 shape
#k_x_train = np.array(k_x_train).reshape(k_x_train.shape[0], k_x_train.shape[1], 1)
k_x_train = k_x_train
k_y_train = k_y_train


# In[ ]:


print(k_x_train)


# In[ ]:


plt.hist(k_y_train, orientation = 'vertical', histtype = 'bar', color = 'blue')
plt.show() 


# In[ ]:


k_x_train = k_x_train.values
X_test = X_test.values


# In[ ]:


def train_model(X, X_test, y, params, folds, model_type='lgb', eval_type='regression'):
    oof = np.zeros(X.shape[0])
    predictions = np.zeros(X_test.shape[0])
    scores = []
    for fold_n, (trn_idx, val_idx) in enumerate(folds.split(X, y)):
        print('Fold', fold_n, 'started at', time.ctime())
        
        if model_type == 'lgb':
            trn_data = lgb.Dataset(X[trn_idx], y[trn_idx])
            val_data = lgb.Dataset(X[val_idx], y[val_idx])
           # trn_data = lgb.Dataset(X[trn_idx].iloc[trn_index],label=target.iloc[trn_index])
           # val_data = lgb.Dataset(X[val_idx].iloc[val_index],label=target.iloc[val_index])
                       
            clf = lgb.train(params, trn_data, num_boost_round=20000, 
                            valid_sets=[trn_data, val_data], 
                            verbose_eval=100, early_stopping_rounds=300)
            oof[val_idx] = clf.predict(X[val_idx], num_iteration=clf.best_iteration)
            predictions += clf.predict(X_test, num_iteration=clf.best_iteration) / folds.n_splits
        
        if model_type == 'xgb':
            trn_data = xgb.DMatrix(X[trn_idx], y[trn_idx])
            val_data = xgb.DMatrix(X[val_idx], y[val_idx])
            watchlist = [(trn_data, 'train'), (val_data, 'valid_data')]
            clf = xgb.train(dtrain=trn_data, num_boost_round=20000, 
                            evals=watchlist, early_stopping_rounds=200, 
                            verbose_eval=100, params=params)
            oof[val_idx] = clf.predict(xgb.DMatrix(X[val_idx]), ntree_limit=clf.best_ntree_limit)
            predictions += clf.predict(xgb.DMatrix(X_test), ntree_limit=clf.best_ntree_limit) / folds.n_splits
        
        if (model_type == 'cat') and (eval_type == 'regression'):
            clf = CatBoostRegressor(iterations=20000, eval_metric='MAE', **params)
            clf.fit(X[trn_idx], y[trn_idx], 
                    eval_set=(X[val_idx], y[val_idx]),
                    cat_features=[], use_best_model=True, verbose=100)
            oof[val_idx] = clf.predict(X[val_idx])
            predictions += clf.predict(X_test) / folds.n_splits
            
        if (model_type == 'cat') and (eval_type == 'binary'):
            clf = CatBoostClassifier(iterations=20000, eval_metric='Logloss', **params)
            clf.fit(X[trn_idx], y[trn_idx], 
                    eval_set=(X[val_idx], y[val_idx]),
                    cat_features=[], use_best_model=True, verbose=100)
            oof[val_idx] = clf.predict_proba(X[val_idx])[:,1]
            predictions += clf.predict_proba(X_test)[:,1] / folds.n_splits
        print(predictions)
        if eval_type == 'regression':
            scores.append(mean_squared_error(oof[val_idx], y[val_idx])**0.5)
        if eval_type == 'multiclass':
            scores.append(log_loss(y[val_idx], oof[val_idx]))
        
    print('CV mean score: {0:.4f}, std: {1:.4f}.'.format(np.mean(scores), np.std(scores)))
  
    return oof, predictions, scores


# In[ ]:


#### lgb
lgb_params = {
             'num_leaves': 18,    #63,20,18
             'min_data_in_leaf': 18,   #32,20
             'objective':'multiclass',
             'num_class': 4,
             'max_depth':3,#4
             'learning_rate': 0.01,
             "min_child_samples": 20,
             "boosting": "gbdt",
             "feature_fraction": 0.95,
             "bagging_freq": 1,
             "bagging_fraction": 0.95,
             "bagging_seed": 11,
            # "metric": 'mae',
             'lambda_l1': 1,  #1
             'lambda_l2': 0.5,  
             "verbosity": -1,
             'seed':2021}
folds = KFold(n_splits=5, shuffle=True, random_state=2021)
#X_ntrain = ntrain[fea_cols].values
# 生成数据和标签
print('='*10,'回归模型','='*10)
oof_lgb , predictions_lgb , scores_lgb  = train_model(k_x_train , X_test, k_y_train, params=lgb_params, folds=folds, model_type='lgb', eval_type='multiclass')


# In[ ]:


sub_df=pd.DataFrame()
sub_df["target"] = predictions_lgb
sub_df.to_csv('predictions_lgb.csv', index=False)


# In[ ]:


oof_lgb  = pd.DataFrame(oof_lgb)
predictions_lgb  = pd.DataFrame(predictions_lgb)
oof_lgb.to_csv('oof_lgb.csv',header=None,index=False)
predictions_lgb.to_csv('predictions_lgb.csv',header=None,index=False)
predictions_lgb.to_csv('predictions_lgb_2.csv',index=False)


# In[ ]:


predictions_lgb_R = (predictions_lgb+0.5).round(0)
predictions_lgb_R.to_csv('predictions_lgb_R.csv',index=False)




#!/usr/bin/env python
# coding: utf-8


# 导入第三方包
import pandas as pd
import numpy as np

import lightgbm as lgb

from sklearn.model_selection import KFold
from sklearn.metrics import f1_score, roc_auc_score

import warnings
warnings.filterwarnings('ignore')



# 读取数据集，具体下载方式可见操作手册
train = pd.read_csv('train.csv')
test = pd.read_csv('test.csv')

sample_submit = pd.read_csv('sample_submit.csv')


# 训练数据及测试数据准备
all_cols = [f for f in train.columns if f not in ['customer_id','loan_default']]

x_train = train[all_cols]
x_test = test[all_cols]

y_train = train['loan_default']


# 作为baseline部分仅使用经典的**LightGBM**作为训练模型，我们还能尝试**XGBoost、CatBoost和NN（神经网络）**
def cv_model(clf, train_x, train_y, test_x, clf_name='lgb'):
    folds = 5
    seed = 2021
    kf = KFold(n_splits=folds, shuffle=True, random_state=seed)

    train = np.zeros(train_x.shape[0])
    test = np.zeros(test_x.shape[0])

    cv_scores = []

    for i, (train_index, valid_index) in enumerate(kf.split(train_x, train_y)):
        print('************************************ {} ************************************'.format(str(i+1)))
        trn_x, trn_y, val_x, val_y = train_x.iloc[train_index], train_y[train_index], train_x.iloc[valid_index], train_y[valid_index]

        train_matrix = clf.Dataset(trn_x, label=trn_y)
        valid_matrix = clf.Dataset(val_x, label=val_y)

        params = {
            'boosting_type': 'gbdt',
            'objective': 'binary',
            'metric': 'auc',
            'min_child_weight': 5,
            'num_leaves': 2 ** 7,
            'lambda_l2': 10,
            'feature_fraction': 0.9,
            'bagging_fraction': 0.9,
            'bagging_freq': 4,
            'learning_rate': 0.01,
            'seed': 2021,
            'nthread': 28,
            'n_jobs':-1,
            'silent': True,
            'verbose': -1,
        }

        model = clf.train(params, train_matrix, 50000, valid_sets=[train_matrix, valid_matrix], verbose_eval=500,early_stopping_rounds=200)
        val_pred = model.predict(val_x, num_iteration=model.best_iteration)
        test_pred = model.predict(test_x, num_iteration=model.best_iteration)

        # print(list(sorted(zip(features, model.feature_importance("gain")), key=lambda x: x[1], reverse=True))[:20])

        train[valid_index] = val_pred
        test += test_pred / kf.n_splits
        cv_scores.append(roc_auc_score(val_y, val_pred))
        
        print(cv_scores)
       
    print("%s_scotrainre_list:" % clf_name, cv_scores)
    print("%s_score_mean:" % clf_name, np.mean(cv_scores))
    print("%s_score_std:" % clf_name, np.std(cv_scores))
    return train, test



lgb_train, lgb_test = cv_model(lgb, x_train, y_train, x_test)


# 预测结果
sample_submit['loan_default'] = lgb_test
sample_submit['loan_default'] = sample_submit['loan_default'].apply(lambda x:1 if x>0.25 else 0).values
sample_submit.to_csv('baseline_result.csv', index=False)







#!/usr/bin/env python
# coding: utf-8

# In[ ]:


# 乘客时序数据
from pycaret.datasets import get_data
# 下载路径：https://raw.githubusercontent.com/sktime/sktime/main/sktime/datasets/data/Airline/Airline.csv
data = get_data('./datasets/airline')
# data = get_data('airline')


# In[ ]:


import pandas as pd
data['Date'] = pd.to_datetime(data['Date'])
# 并将Date设置为列号
data.set_index('Date', inplace=True)


# In[ ]:


from pycaret.time_series import TSForecastingExperiment
s = TSForecastingExperiment()
# fh: 用于预测的预测范围。默认值设置为1，即预测前方一点。,fold: 交叉验证中折数
s.setup(data, fh = 3, fold = 5, session_id = 0, verbose = False)
# from pycaret.time_series import *
# s = setup(data, fh = 3, fold = 5, session_id = 0)


# In[ ]:


#模型训练与评估
best = s.compare_models()



# In[ ]:


#数据展示

# jupyter环境下交互可视化展示
# plot参数支持：
# - 'ts' - Time Series Plot
# - 'train_test_split' - Train Test Split
# - 'cv' - Cross Validation
# - 'acf' - Auto Correlation (ACF)
# - 'pacf' - Partial Auto Correlation (PACF)
# - 'decomp' - Classical Decomposition
# - 'decomp_stl' - STL Decomposition
# - 'diagnostics' - Diagnostics Plot
# - 'diff' - Difference Plot
# - 'periodogram' - Frequency Components (Periodogram)
# - 'fft' - Frequency Components (FFT)
# - 'ccf' - Cross Correlation (CCF)
# - 'forecast' - "Out-of-Sample" Forecast Plot
# - 'insample' - "In-Sample" Forecast Plot
# - 'residuals' - Residuals Plot
# s.plot_model(best, plot = 'forecast', data_kwargs = {'fh' : 24})


# In[ ]:


#数据预测
# 使模型拟合包括测试样本在内的完整数据集
final_best = s.finalize_model(best)
s.predict_model(best, fh = 24)
s.save_model(final_best, 'final_best_model')




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import pandas as pd
import numpy as np

train_data = pd.read_csv('用户新增预测挑战赛公开数据/train.csv')
test_data = pd.read_csv('用户新增预测挑战赛公开数据/test.csv')

train_data['common_ts'] = pd.to_datetime(train_data['common_ts'], unit='ms')
test_data['common_ts'] = pd.to_datetime(test_data['common_ts'], unit='ms')


# In[ ]:


def udmap_onethot(d):
    v = np.zeros(9)
    if d == 'unknown':
        return v
    
    d = eval(d)
    for i in range(1, 10):
        if 'key' + str(i) in d:
            v[i-1] = d['key' + str(i)]
            
    return v

train_udmap_df = pd.DataFrame(np.vstack(train_data['udmap'].apply(udmap_onethot)))
test_udmap_df = pd.DataFrame(np.vstack(test_data['udmap'].apply(udmap_onethot)))

train_udmap_df.columns = ['key' + str(i) for i in range(1, 10)]
test_udmap_df.columns = ['key' + str(i) for i in range(1, 10)]


# In[ ]:


train_data = pd.concat([train_data, train_udmap_df], axis=1)
test_data = pd.concat([test_data, test_udmap_df], axis=1)


# In[ ]:


train_data['eid_freq'] = train_data['eid'].map(train_data['eid'].value_counts())
test_data['eid_freq'] = test_data['eid'].map(train_data['eid'].value_counts())

train_data['eid_mean'] = train_data['eid'].map(train_data.groupby('eid')['target'].mean())
test_data['eid_mean'] = test_data['eid'].map(train_data.groupby('eid')['target'].mean())


# In[ ]:


train_data['udmap_isunknown'] = (train_data['udmap'] == 'unknown').astype(int)
test_data['udmap_isunknown'] = (test_data['udmap'] == 'unknown').astype(int)


# In[ ]:


train_data['common_ts_hour'] = train_data['common_ts'].dt.hour
test_data['common_ts_hour'] = test_data['common_ts'].dt.hour


# In[ ]:


import lightgbm as lgb

clf = lgb.LGBMClassifier()
clf.fit(
    train_data.drop(['udmap', 'common_ts', 'uuid', 'target'], axis=1),
    train_data['target']
)


# In[ ]:


pd.DataFrame({
    'uuid': test_data['uuid'],
    'target': clf.predict(test_data.drop(['udmap', 'common_ts', 'uuid'], axis=1))
}).to_csv('submit.csv', index=None)


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from autogluon.tabular import TabularDataset, TabularPredictor

import numpy as np

train_data = TabularDataset('https://autogluon.s3.amazonaws.com/datasets/Inc/train.csv')
subsample_size = 1000  # subsample subset of data for faster demo, try setting this to much larger values
train_data = train_data.sample(n=subsample_size, random_state=0)
print(train_data.head())

label = 'occupation'
print("Summary of occupation column: \n", train_data['occupation'].describe())

test_data = TabularDataset('https://autogluon.s3.amazonaws.com/datasets/Inc/test.csv')
y_test = test_data[label]
test_data_nolabel = test_data.drop(columns=[label])  # delete label column

metric = 'accuracy' # we specify eval-metric just for demo (unnecessary as it's the default)


# In[ ]:


from autogluon.common import space

nn_options = {  # specifies non-default hyperparameter values for neural network models
    'num_epochs': 10,  # number of training epochs (controls training time of NN models)
    'learning_rate': space.Real(1e-4, 1e-2, default=5e-4, log=True),  # learning rate used in training (real-valued hyperparameter searched on log-scale)
    'activation': space.Categorical('relu', 'softrelu', 'tanh'),  # activation function used in NN (categorical hyperparameter, default = first entry)
    'dropout_prob': space.Real(0.0, 0.5, default=0.1),  # dropout probability (real-valued hyperparameter)
}

gbm_options = {  # specifies non-default hyperparameter values for lightGBM gradient boosted trees
    'num_boost_round': 100,  # number of boosting rounds (controls training time of GBM models)
    'num_leaves': space.Int(lower=26, upper=66, default=36),  # number of leaves in trees (integer hyperparameter)
}

hyperparameters = {  # hyperparameters of each model type
                   'GBM': gbm_options,
                   'NN_TORCH': nn_options,  # NOTE: comment this line out if you get errors on Mac OSX
                  }  # When these keys are missing from hyperparameters dict, no models of that type are trained

time_limit = 2*60  # train various models for ~2 min
num_trials = 5  # try at most 5 different hyperparameter configurations for each type of model
search_strategy = 'auto'  # to tune hyperparameters using random search routine with a local scheduler

hyperparameter_tune_kwargs = {  # HPO is not performed unless hyperparameter_tune_kwargs is specified
    'num_trials': num_trials,
    'scheduler' : 'local',
    'searcher': search_strategy,
}  # Refer to TabularPredictor.fit docstring for all valid values

predictor = TabularPredictor(label=label, eval_metric=metric).fit(
    train_data,
    time_limit=time_limit,
    hyperparameters=hyperparameters,
    hyperparameter_tune_kwargs=hyperparameter_tune_kwargs,
)


# In[ ]:


y_pred = predictor.predict(test_data_nolabel)
print("Predictions:  ", list(y_pred)[:5])
perf = predictor.evaluate(test_data, auxiliary_metrics=False)


# In[ ]:


results = predictor.fit_summary()


# In[ ]:





# In[ ]:


#使用 stacking/bagging 进行模型集成
label = 'class'  # Now lets predict the "class" column (binary classification)
test_data_nolabel = test_data.drop(columns=[label])
y_test = test_data[label]
save_path = 'agModels-predictClass'  # folder where to store trained models

predictor = TabularPredictor(label=label, eval_metric=metric).fit(train_data,
    num_bag_folds=5, num_bag_sets=1, num_stack_levels=1,
    hyperparameters = {'NN_TORCH': {'num_epochs': 2}, 'GBM': {'num_boost_round': 20}},  # last  argument is just for quick demo here, omit it in real applications
)


# In[ ]:


# Lets also specify the "f1" metric
predictor = TabularPredictor(label=label, eval_metric='f1', path=save_path).fit(
    train_data, auto_stack=True,
    time_limit=30, hyperparameters={'FASTAI': {'num_epochs': 10}, 'GBM': {'num_boost_round': 200}}  # last 2 arguments are for quick demo, omit them in real applications
)
predictor.leaderboard(test_data)


# In[ ]:


#决策阈值校准
'''
通过将预测决策阈值调整为默认值 0.5 以外的值，可以在二元分类中实现指标得分的大幅提升，例如和"f1"。"balanced_accuracy"calibrate_decision_threshold

"f1"以下是校准和未校准决策阈值的情况下在测试数据上取得的分数的示例：'''
print(f'Prior to calibration (predictor.decision_threshold={predictor.decision_threshold}):')
scores = predictor.evaluate(test_data)

calibrated_decision_threshold = predictor.calibrate_decision_threshold()
predictor.set_decision_threshold(calibrated_decision_threshold)

print(f'After calibration (predictor.decision_threshold={predictor.decision_threshold}):')
scores_calibrated = predictor.evaluate(test_data)


# In[ ]:


#默认情况下，predict()将predict_proba()使用 AutoGluon 认为最准确的模型，该模型通常是许多单个模型的集合。以下是如何查看这是哪个模型：
predictor.model_best


# In[ ]:


#在决定使用哪个模型之前，让我们评估 AutoGluon 之前在我们的测试数据上训练的所有模型：
predictor.leaderboard(test_data)
predictor.leaderboard(extra_info=True)
predictor.leaderboard(test_data, extra_metrics=['accuracy', 'balanced_accuracy', 'log_loss'])
i = 0  # index of model to use
model_to_use = predictor.model_names()[i]
model_pred = predictor.predict(datapoint, model=model_to_use)
print("Prediction from %s model: %s" % (model_to_use, model_pred.iloc[0]))


# In[ ]:


#我们可以轻松访问有关训练预测器或特定模型的各种信息：
all_models = predictor.model_names()
model_to_use = all_models[i]
specific_model = predictor._trainer.load_model(model_to_use)

# Objects defined below are dicts of various information (not printed here as they are quite large):
model_info = specific_model.get_info()
predictor_information = predictor.info()


# In[ ]:


y_pred_proba = predictor.predict_proba(test_data_nolabel)
perf = predictor.evaluate_predictions(y_true=y_test, y_pred=y_pred_proba)
perf = predictor.evaluate(test_data)
predictor.feature_importance(test_data)


# In[ ]:


'''
推理速度作为拟合约束
如果您在拟合预测器之前知道延迟约束，则可以将其明确指定为拟合参数。AutoGluon 随后将以尝试满足约束的方式自动训练模型。

此约束包含两个部分：infer_limit和infer_limit_batch_size：

infer_limit是预测 1 行数据所需的时间（以秒为单位）。例如，infer_limit=0.05表示每行数据 50 毫秒，或每秒 20 行的吞吐量。

infer_limit_batch_size是计算每行速度时一次传递的要预测的行数。这非常重要，因为infer_limit_batch_size=1（在线推理）非常不理想，因为各种操作无论数据大小都有固定的成本开销。如果您可以批量传递测试数据，则应指定infer_limit_batch_size=10000。
'''


# In[ ]:


# At most 0.05 ms per row (20000 rows per second throughput)
infer_limit = 0.00005
# adhere to infer_limit with batches of size 10000 (batch-inference, easier to satisfy infer_limit)
infer_limit_batch_size = 10000
# adhere to infer_limit with batches of size 1 (online-inference, much harder to satisfy infer_limit)
# infer_limit_batch_size = 1  # Note that infer_limit<0.02 when infer_limit_batch_size=1 can be difficult to satisfy.
predictor_infer_limit = TabularPredictor(label=label, eval_metric=metric).fit(
    train_data=train_data,
    time_limit=30,
    infer_limit=infer_limit,
    infer_limit_batch_size=infer_limit_batch_size,
)

# NOTE: If bagging was enabled, it is important to call refit_full at this stage.
#  infer_limit assumes that the user will call refit_full after fit.
# predictor_infer_limit.refit_full()

# NOTE: To align with inference speed calculated during fit, models must be persisted.
predictor_infer_limit.persist()
# Below is an optimized version that only persists the minimum required models for prediction.
# predictor_infer_limit.persist('best')

predictor_infer_limit.leaderboard()


# In[ ]:


test_data_batch = test_data.sample(infer_limit_batch_size, replace=True, ignore_index=True)

import time
time_start = time.time()
predictor_infer_limit.predict(test_data_batch)
time_end = time.time()

infer_time_per_row = (time_end - time_start) / len(test_data_batch)
rows_per_second = 1 / infer_time_per_row
infer_time_per_row_ratio = infer_time_per_row / infer_limit
is_constraint_satisfied = infer_time_per_row_ratio <= 1

print(f'Model is able to predict {round(rows_per_second, 1)} rows per second. (User-specified Throughput = {1 / infer_limit})')
print(f'Model uses {round(infer_time_per_row_ratio * 100, 1)}% of infer_limit time per row.')
print(f'Model satisfies inference constraint: {is_constraint_satisfied}')


# In[ ]:


'''
更快的预设或超参数
fit()如果您知道推理延迟或内存一开始会成为问题，那么您可以相应地调整训练过程以确保不会产生笨重的模型，而不是试图在预测时加速繁琐的训练模型。

一种选择是指定更轻量级的presets：
'''


# In[ ]:


presets = ['good_quality', 'optimize_for_deployment']
predictor_light = TabularPredictor(label=label, eval_metric=metric).fit(train_data, presets=presets, time_limit=30)


# In[ ]:


#另一种选择是指定更轻量的超参数：
predictor_light = TabularPredictor(label=label, eval_metric=metric).fit(train_data, hyperparameters='very_light', time_limit=30)


# In[ ]:


#最后，你也可以完全排除某些难以操作的模型进行训练。下面我们排除那些速度较慢的模型（K 近邻、神经网络）：
excluded_model_types = ['KNN', 'NN_TORCH']
predictor_light = TabularPredictor(label=label, eval_metric=metric).fit(train_data, excluded_model_types=excluded_model_types, time_limit=30)


# In[ ]:


'''
{
	'NN_TORCH': [{}],
	'GBM': [{'extra_trees': True, 'ag_args': {'name_suffix': 'XT'}}, {}, {'learning_rate': 0.03, 'num_leaves': 128, 'feature_fraction': 0.9, 'min_data_in_leaf': 3, 'ag_args': {'name_suffix': 'Large', 'priority': 0, 'hyperparameter_tune_kwargs': None}}],
	'CAT': [{}],
	'XGB': [{}],
	'FASTAI': [{}],
	'RF': [{'criterion': 'gini', 'ag_args': {'name_suffix': 'Gini', 'problem_types': ['binary', 'multiclass']}}, {'criterion': 'entropy', 'ag_args': {'name_suffix': 'Entr', 'problem_types': ['binary', 'multiclass']}}, {'criterion': 'squared_error', 'ag_args': {'name_suffix': 'MSE', 'problem_types': ['regression', 'quantile']}}],
	'XT': [{'criterion': 'gini', 'ag_args': {'name_suffix': 'Gini', 'problem_types': ['binary', 'multiclass']}}, {'criterion': 'entropy', 'ag_args': {'name_suffix': 'Entr', 'problem_types': ['binary', 'multiclass']}}, {'criterion': 'squared_error', 'ag_args': {'name_suffix': 'MSE', 'problem_types': ['regression', 'quantile']}}],
	'KNN': [{'weights': 'uniform', 'ag_args': {'name_suffix': 'Unif'}}, {'weights': 'distance', 'ag_args': {'name_suffix': 'Dist'}}],
}
'''


# In[ ]:


'''
如果遇到内存
为了减少训练期间的内存使用量，您可以单独尝试以下每种策略或将它们组合起来（但这可能会损害准确性）：

在 中fit()，集合（或这些模型的某些子集）。excluded_model_types = ['KNN', 'XT' ,'RF']

尝试不同presets的fit()。

在fit()，设置或。hyperparameters = 'light'hyperparameters = 'very_light'

表中的文本字段需要大量内存来进行 N-gram 特征化。为了缓解这种情况fit()，您可以：(1) 添加'ignore_text'到presets列表中（忽略文本特征），或 (2) 指定参数：

'''
from sklearn.feature_extraction.text import CountVectorizer
from autogluon.features.generators import AutoMLPipelineFeatureGenerator
feature_generator = AutoMLPipelineFeatureGenerator(vectorizer=CountVectorizer(min_df=30, ngram_range=(1, 3), max_features=MAX_NGRAM, dtype=np.uint8))
#例如使用（尝试 10000 以下的各种值来减少用于表示每个文本字段的 N-gram 特征的数量）MAX_NGRAM = 1000


# In[ ]:


'''
如果遇到磁盘空间
为了减少磁盘使用量，您可以单独尝试以下每种策略或尝试它们的组合：

确保删除predictor.path之前运行的所有文件夹！如果多次fit()调用，这些文件夹可能会占用您的可用空间。如果您未指定，AutoGluon 仍会自动将其模型保存到名为“AutogluonModels/ag-[TIMESTAMP]”的文件夹中，其中 TIMESTAMP 记录了调用时间，因此如果可用空间不足，请确保也删除这些文件夹。fit()pathfit()

调用predictor.save_space()以删除期间生成的辅助文件fit()。

如果您只打算使用此预测器进行推理，请调用（将删除非预测相关功能所需的文件，如）。predictor.delete_models(models_to_keep='best', dry_run=False)fit_summary

在 中fit()，你可以添加'optimize_for_deployment'到presets列表中，训练完成后将自动调用前两种策略。

上述减少内存使用量的策略大多数也会减少磁盘使用量（但可能会损害准确性）。

'''




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from sklearn.decomposition import PCA #主成分分析法
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt 
from sklearn.ensemble import GradientBoostingClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split
from sklearn.linear_model import Ridge
import os
import math
import time
import random
import datetime
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import tensorflow as tf
import tensorflow.keras as K
from tensorflow.keras import Sequential, utils, regularizers, Model, Input
from tensorflow.keras.layers import Flatten, Dense, Conv1D, MaxPool1D, Dropout, AvgPool1D
from imblearn.over_sampling import SMOTE
from sklearn.model_selection import KFold
from sklearn.preprocessing import OneHotEncoder
from sklearn.metrics import roc_auc_score
import json
import warnings
from sklearn import preprocessing
import lightgbm as lgb  
import pickle  
from sklearn.model_selection import train_test_split  
from preprocessing import filterData, delData, fillNanList, dropDuplicate,\
    handleOutlier, minMaxScale, cate2Num,standardizeData,\
    discrete, tran_math_function, minMaxScale, standardizeData,\
    onehot_map, map_dict_tran, binary_map, pca_selection,dfs_feature,\
    continue_time, discrete_time, statistics_time
from sklearn.metrics import roc_auc_score, roc_curve,mean_squared_error
from sklearn.linear_model import LogisticRegression
from sklearn.svm import SVC, LinearSVC
from sklearn.ensemble import RandomForestClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn.linear_model import Perceptron
from sklearn.linear_model import SGDClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.model_selection import GridSearchCV
from sklearn import metrics
pd.set_option('display.max_columns',None)
warnings.filterwarnings("ignore")


# In[ ]:


# 导入数据  
print("Loading Data ... ")  
new_train_data = pd.read_csv('datasets/@huangzx23#fd1a8eafd2b79230d7853599248258a8/train.csv')
new_test_data = pd.read_csv('datasets/@huangzx23#fd1a8eafd2b79230d7853599248258a8/test.csv')
new_train_data = new_train_data.drop(['cell_most','is_user_new','flow_2g','flow_3g','flow_4g','is_voice_stable','net_speed','is_net_stable','is_net_cover','service_type','order_id','service_type','price','msisdn','cell_longest'], axis=1)
new_test_data = new_test_data.drop(['cell_most','is_user_new','flow_2g','flow_3g','flow_4g','is_voice_stable','net_speed','is_net_stable','is_net_cover','service_type','order_id','service_type','price','msisdn','cell_longest'], axis=1)


# In[ ]:


#合并字段
data_all = pd.concat([new_train_data.assign(is_train = 1),new_test_data.assign(is_train = 0)]) #合并train和test，并且用is_train进行标记
train = data_all['is_train'] == 1##提前进行标记
test  = data_all['is_train'] == 0
print('数据全集量是',len(data_all))
train_count = len(data_all[train])
print('训练集样本量是',train_count)
test_count = len(data_all[test])
print('测试集样本量是',test_count)
print('样本比例为：', train_count/test_count)


# In[ ]:


data_all.info(verbose=True, null_counts=True)


# In[ ]:


pd.set_option('display.max_rows',None)
data_all['user_sex'].value_counts()
data_all['complaint_theme']=data_all['complaint_theme'].fillna('0')


# In[ ]:


fact_list=data_all['user_dinner'].value_counts()[0:30]
data_all['user_dinner']=data_all['user_dinner'].apply(lambda x:'其他' if x not in fact_list else x)


# In[ ]:


label = preprocessing.LabelEncoder()
label.fit(np.unique(list(data_all['complaint_theme'].values)))
data_all['complaint_theme']=label.transform(list(data_all['complaint_theme'].values))


# In[ ]:


#######KNN填充缺失值
from sklearn.impute import KNNImputer
imputer = KNNImputer(n_neighbors=10)
data_all[['talklen_2g','talklen_3g','volte_talklen_out','volte_talklen_in','lte_uhttp_base_003_week','talk_2g_count_in','talk_3g_count_in','talk_volte_count_in','talk_2g_count_out','talk_3g_count_out','talk_volte_count_out','online_2g','online_3g','online_4g','online_5g','userp_count_003_2g','userp_count_003_3g','userp_count_003_4g','userp_count_006_2g','userp_count_006_3g','talk_2g_count_in','talk_3g_count_in','talk_volte_count_in','talk_2g_count_out','talk_3g_count_out','talk_volte_count_out','online_2g','online_3g','online_4g','online_5g','userp_count_003_2g','userp_count_003_3g','userp_count_003_4g','userp_count_006_2g','userp_count_006_3g','userp_count_006_4g','userp_count_010_2g','userp_count_010_3g','userp_count_010_4g','userp_count_009_2g','userp_count_009_3g','userp_count_009_4g','lte_uhttp_base_008_week','lte_uhttp_base_009_week','lte_uhttp_base_010_week','lte_uhttp_base_013_week','lte_uhttp_base_004_week','lte_uhttp_base_005_week','lte_uhttp_base_007_week','lte_uhttp_base_018_week','lte_uhttp_base_019_week','lte_uhttp_base_021_week','lte_uhttp_base_020_week','lte_uhttp_base_022_week','lte_uhttp_base_023_week','lte_uhttp_base_024_week','lte_uhttp_base_025_week','lte_uhttp_base_026_week','lte_uhttp_base_027_week','lte_uhttp_base_028_week','lte_uhttp_base_029_week','lte_uhttp_base_035_week','lte_uhttp_base_044_week','lte_uhttp_base_045_week','lte_uhttp_base_049_week','lte_uhttp_base_050_week','lte_uhttp_base_051_week','lte_uhttp_base_052_week','lte_uhttp_base_053_week','lte_uhttp_base_054_week','lte_ustreaming_base_007_week','lte_ustreaming_base_008_week','lte_ustreaming_base_009_week','lte_ustreaming_base_003_week','lte_ustreaming_base_004_week','lte_ustreaming_base_005_week','lte_ustreaming_base_006_week','lte_ustreaming_base_010_week','lte_ustreaming_base_011_week','lte_ustreaming_base_012_week','lte_ustreaming_base_013_week', 'lte_ustreaming_base_014_week','lte_ustreaming_base_015_week','lte_ustreaming_base_016_week']] = imputer.fit_transform(data_all[['talklen_2g','talklen_3g','volte_talklen_out','volte_talklen_in','lte_uhttp_base_003_week','talk_2g_count_in','talk_3g_count_in','talk_volte_count_in','talk_2g_count_out','talk_3g_count_out','talk_volte_count_out','online_2g','online_3g','online_4g','online_5g','userp_count_003_2g','userp_count_003_3g','userp_count_003_4g','userp_count_006_2g','userp_count_006_3g','talk_2g_count_in','talk_3g_count_in','talk_volte_count_in','talk_2g_count_out','talk_3g_count_out','talk_volte_count_out','online_2g','online_3g','online_4g','online_5g','userp_count_003_2g','userp_count_003_3g','userp_count_003_4g','userp_count_006_2g','userp_count_006_3g','userp_count_006_4g','userp_count_010_2g','userp_count_010_3g','userp_count_010_4g','userp_count_009_2g','userp_count_009_3g','userp_count_009_4g','lte_uhttp_base_008_week','lte_uhttp_base_009_week','lte_uhttp_base_010_week','lte_uhttp_base_013_week','lte_uhttp_base_004_week','lte_uhttp_base_005_week','lte_uhttp_base_007_week','lte_uhttp_base_018_week','lte_uhttp_base_019_week','lte_uhttp_base_021_week','lte_uhttp_base_020_week','lte_uhttp_base_022_week','lte_uhttp_base_023_week','lte_uhttp_base_024_week','lte_uhttp_base_025_week','lte_uhttp_base_026_week','lte_uhttp_base_027_week','lte_uhttp_base_028_week','lte_uhttp_base_029_week','lte_uhttp_base_035_week','lte_uhttp_base_044_week','lte_uhttp_base_045_week','lte_uhttp_base_049_week','lte_uhttp_base_050_week','lte_uhttp_base_051_week','lte_uhttp_base_052_week','lte_uhttp_base_053_week','lte_uhttp_base_054_week','lte_ustreaming_base_007_week','lte_ustreaming_base_008_week','lte_ustreaming_base_009_week','lte_ustreaming_base_003_week','lte_ustreaming_base_004_week','lte_ustreaming_base_005_week','lte_ustreaming_base_006_week','lte_ustreaming_base_010_week','lte_ustreaming_base_011_week','lte_ustreaming_base_012_week','lte_ustreaming_base_013_week', 'lte_ustreaming_base_014_week','lte_ustreaming_base_015_week','lte_ustreaming_base_016_week']])
print('train:success!')


# In[ ]:


new_train_data = data_all[data_all['is_train']== 1]
new_test_data  = data_all[data_all['is_train']== 0]

#new_train=new_train.astype('float64')
#new_test=new_test.astype('float64')

new_train_data = new_train_data.drop(['is_train'], axis=1)
new_test_data = new_test_data.drop(['value','is_train'], axis=1)

train_data=new_train_data
test_data=new_test_data


# In[ ]:


train_data.info(verbose=True, null_counts=True)
test_data.info(verbose=True, null_counts=True)


# In[ ]:


train_data_bak=train_data.copy()
test_data_bak=test_data.copy()


# In[ ]:


train_data.info(verbose=True, null_counts=True)
test_data.info(verbose=True, null_counts=True)


# In[ ]:


print('训练集样本量是',len(train_data))
print('测试集样本量是',len(test_data))

train_data['talk_count'] =np.sum(train_data[['talk_2g_count_in','talk_3g_count_in','talk_volte_count_in','talk_2g_count_out','talk_3g_count_out','talk_volte_count_out']],axis=1) #######################考虑分箱

train_data['online_all'] =np.sum(train_data[['online_2g','online_3g','online_4g','online_4g']],axis=1)   #######################考虑分箱

train_data['USERP_COUNT_1_1'] =np.sum(train_data[['userp_count_003_2g','userp_count_003_3g','userp_count_003_4g']],axis=1)   #######################考虑分箱

train_data['USERP_COUNT_1_1'] =np.sum(train_data[['userp_count_006_2g','userp_count_006_3g','userp_count_006_4g']],axis=1)  #######################考虑分箱

#train_data['USERP_COUNT_2'] =np.sum(train_data[['userp_count_010_2g','userp_count_010_3g','userp_count_010_4g',]],axis=1) / np.sum(train_data[['userp_count_009_2g','userp_count_009_3g','userp_count_009_4g']],axis=1) 

train_data['USERP_COUNT_2_1'] =train_data['userp_count_010_2g']/train_data['userp_count_009_2g']
train_data['USERP_COUNT_2_2'] =train_data['userp_count_010_3g']/train_data['userp_count_009_3g']
train_data['USERP_COUNT_2_3'] =train_data['userp_count_010_4g']/train_data['userp_count_009_4g']


# In[ ]:


print('训练集样本量是',len(test_data))
print('测试集样本量是',len(test_data))

test_data['talk_count'] =np.sum(test_data[['talk_2g_count_in','talk_3g_count_in','talk_volte_count_in','talk_2g_count_out','talk_3g_count_out','talk_volte_count_out']],axis=1) #######################考虑分箱

test_data['online_all'] =np.sum(test_data[['online_2g','online_3g','online_4g','online_4g']],axis=1)   #######################考虑分箱

test_data['USERP_COUNT_1_1'] =np.sum(test_data[['userp_count_003_2g','userp_count_003_3g','userp_count_003_4g']],axis=1)   #######################考虑分箱

test_data['USERP_COUNT_1_1'] =np.sum(test_data[['userp_count_006_2g','userp_count_006_3g','userp_count_006_4g']],axis=1)  #######################考虑分箱


# In[ ]:


train_data=train_data.drop(['is_lucknumber','talklen_2g','talklen_3g','volte_talklen_out','volte_talklen_in','lte_uhttp_base_003_week','talk_2g_count_in','talk_3g_count_in','talk_volte_count_in','talk_2g_count_out','talk_3g_count_out','talk_volte_count_out','online_2g','online_3g','online_4g',  
                            'online_5g','userp_count_003_2g','userp_count_003_3g','userp_count_003_4g','userp_count_006_2g','userp_count_006_3g','talk_2g_count_in','talk_3g_count_in','talk_volte_count_in','talk_2g_count_out','talk_3g_count_out','talk_volte_count_out','online_2g','online_3g',  
                            'online_4g','online_5g','userp_count_003_2g','userp_count_003_3g','userp_count_003_4g','userp_count_006_2g','userp_count_006_3g','userp_count_006_4g','userp_count_010_2g','userp_count_010_3g','userp_count_010_4g','userp_count_009_2g','userp_count_009_3g','userp_count_009_4g','lte_uhttp_base_008_week','lte_uhttp_base_009_week','lte_uhttp_base_010_week','lte_uhttp_base_013_week','lte_uhttp_base_004_week','lte_uhttp_base_005_week','lte_uhttp_base_007_week','lte_uhttp_base_018_week','lte_uhttp_base_019_week','lte_uhttp_base_021_week','lte_uhttp_base_020_week','lte_uhttp_base_022_week','lte_uhttp_base_023_week','lte_uhttp_base_024_week','lte_uhttp_base_025_week','lte_uhttp_base_026_week','lte_uhttp_base_027_week','lte_uhttp_base_028_week','lte_uhttp_base_029_week','lte_uhttp_base_035_week','lte_uhttp_base_044_week','lte_uhttp_base_045_week','lte_uhttp_base_049_week','lte_uhttp_base_050_week','lte_uhttp_base_051_week','lte_uhttp_base_052_week','lte_uhttp_base_053_week','lte_uhttp_base_054_week','lte_ustreaming_base_007_week','lte_ustreaming_base_008_week','lte_ustreaming_base_009_week','lte_ustreaming_base_003_week','lte_ustreaming_base_004_week','lte_ustreaming_base_005_week','lte_ustreaming_base_006_week','lte_ustreaming_base_010_week','lte_ustreaming_base_011_week','lte_ustreaming_base_012_week','lte_ustreaming_base_013_week', 'lte_ustreaming_base_014_week','lte_ustreaming_base_015_week','lte_ustreaming_base_016_week','user_status','used_4g','mzie_type','is_volte_sign','is_l900','used_2g','used_5g','cell_type','is_5g','rat','is_mix','miit_net','used_3g','is_dual'], axis=1)

test_data=test_data.drop(['is_lucknumber','talklen_2g','talklen_3g','volte_talklen_out','volte_talklen_in','lte_uhttp_base_003_week','talk_2g_count_in','talk_3g_count_in','talk_volte_count_in','talk_2g_count_out','talk_3g_count_out','talk_volte_count_out','online_2g','online_3g','online_4g','online_5g','userp_count_003_2g','userp_count_003_3g','userp_count_003_4g','userp_count_006_2g','userp_count_006_3g','talk_2g_count_in','talk_3g_count_in','talk_volte_count_in','talk_2g_count_out','talk_3g_count_out','talk_volte_count_out','online_2g','online_3g','online_4g','online_5g','userp_count_003_2g','userp_count_003_3g','userp_count_003_4g','userp_count_006_2g','userp_count_006_3g','userp_count_006_4g','userp_count_010_2g','userp_count_010_3g','userp_count_010_4g','userp_count_009_2g','userp_count_009_3g','userp_count_009_4g','lte_uhttp_base_008_week','lte_uhttp_base_009_week','lte_uhttp_base_010_week','lte_uhttp_base_013_week','lte_uhttp_base_004_week','lte_uhttp_base_005_week','lte_uhttp_base_007_week','lte_uhttp_base_018_week','lte_uhttp_base_019_week','lte_uhttp_base_021_week','lte_uhttp_base_020_week','lte_uhttp_base_022_week','lte_uhttp_base_023_week','lte_uhttp_base_024_week','lte_uhttp_base_025_week','lte_uhttp_base_026_week','lte_uhttp_base_027_week','lte_uhttp_base_028_week','lte_uhttp_base_029_week','lte_uhttp_base_035_week','lte_uhttp_base_044_week','lte_uhttp_base_045_week','lte_uhttp_base_049_week','lte_uhttp_base_050_week','lte_uhttp_base_051_week','lte_uhttp_base_052_week','lte_uhttp_base_053_week','lte_uhttp_base_054_week','lte_ustreaming_base_007_week','lte_ustreaming_base_008_week','lte_ustreaming_base_009_week','lte_ustreaming_base_003_week','lte_ustreaming_base_004_week','lte_ustreaming_base_005_week','lte_ustreaming_base_006_week','lte_ustreaming_base_010_week','lte_ustreaming_base_011_week','lte_ustreaming_base_012_week','lte_ustreaming_base_013_week', 'lte_ustreaming_base_014_week','lte_ustreaming_base_015_week','lte_ustreaming_base_016_week','user_status','used_4g','mzie_type','is_volte_sign','is_l900','used_2g','used_5g','cell_type','is_5g','rat','is_mix','miit_net','used_3g','is_dual'], axis=1)

'''
where_are_inf = np.isinf(train_data)
train_data[where_are_inf] = 0

where_are_inf = np.isinf(test_data)
test_data[where_are_inf] = 0
'''
train_data.to_csv('train_data_count.csv',index=False)
test_data.to_csv('test_data_count.csv',index=False)


# In[ ]:


print(np.isnan(train_data).any())
print(np.isnan(test_data).any())


# In[ ]:


####填充0值
train_data=train_data.fillna(0)
test_data=test_data.fillna(0)

print(np.isnan(train_data).any())
print(np.isnan(test_data).any())


# In[ ]:


# 目前只考虑通过相关性选择特征
#train_data.corr()['value'][abs(train_data.corr()['value'])!='K']


# In[ ]:


# 选择相关性大于0.05的作为候选特征参与训练，并加入我们认为比较重要的特征，总共66个特征参与训练
features = (train_data.corr()['value'][abs(train_data.corr()['value'])!='K']).index
features = features.values.tolist()
features.remove('value')
len(features)


# In[ ]:


'''
train_data['rat']= train_data['rat'].astype('category')
train_data['cell_type']= train_data['cell_type'].astype('category')
'''


# In[ ]:


# 生成数据和标签
target = train_data['value']
train_selected = train_data[features]
test = test_data[features]
feature_importance_df = pd.DataFrame()
oof = np.zeros(len(train_data))
predictions = np.zeros(len(test_data))


# In[ ]:


params = {'num_leaves': 9,
         'min_data_in_leaf': 40,
         'objective': 'regression_l1',
         'max_depth': 16,
         'learning_rate': 0.01,
         'boosting': 'gbdt',
         'bagging_freq': 5,
         'bagging_fraction': 0.8,   # 每次迭代时用的数据比例0.8
         'feature_fraction': 0.8201,# 每次迭代中随机选择80％的参数来建树0.8201
         'bagging_seed': 11,
         'reg_alpha': 1.728910519108444,
         'reg_lambda': 4.9847051755586085,
         'random_state': 42,
         'metric': 'mae',
         'verbosity': -1,
         'subsample': 0.81,
         'min_gain_to_split': 0.01077313523861969,
         'min_child_weight': 19.428902804238373,
         'num_threads': 4,
         'seed':2021}   
kfolds = KFold(n_splits=5,shuffle=True,random_state=15)
predictions = np.zeros(len(test))
cat=['is_5g_dinner','complaint_theme','complaint_channel','fact_name','user_dinner','user_sex','user_lv','is_voice_cover','is_volte','in_net_group','acct_charge_type','gprs_bytes_type','complaint_level'] 
for fold_n,(trn_index,val_index) in enumerate(kfolds.split(train_selected,target)):
    print("fold_n {}".format(fold_n))
    trn_data = lgb.Dataset(train_selected.iloc[trn_index],label=target.iloc[trn_index])
    val_data = lgb.Dataset(train_selected.iloc[val_index],label=target.iloc[val_index])
    num_round=30000
    clf = lgb.train(params, trn_data, num_round, valid_sets = [trn_data, val_data], verbose_eval=1000, early_stopping_rounds = 8000,categorical_feature=cat)
    oof[val_index] = clf.predict(train_selected.iloc[val_index], num_iteration=clf.best_iteration)
    predictions += clf.predict(test,num_iteration=clf.best_iteration)/5
    fold_importance_df = pd.DataFrame()
    fold_importance_df["feature"] = features
    fold_importance_df["importance"] = clf.feature_importance()
    fold_importance_df["fold"] = fold_n + 1
    feature_importance_df = pd.concat([feature_importance_df, fold_importance_df], axis=0)
    print("CV score: {:<8.5f}".format(mean_squared_error(target, oof)**0.5))


# In[ ]:


cols = (feature_importance_df[["feature", "importance"]]
        .groupby("feature")
        .mean()
        .sort_values(by="importance", ascending=False)[:1000].index)
best_features = feature_importance_df.loc[feature_importance_df.feature.isin(cols)]

plt.figure(figsize=(14,26))
sns.barplot(x="importance", y="feature", data=best_features.sort_values(by="importance",ascending=False))
plt.title('LightGBM Features (averaged over folds)')
plt.tight_layout()


# In[ ]:


best_features.to_csv('best_features.csv',index=False)


# In[ ]:


# 计算结果
submision_lgb1=test
submision_lgb1['value']=predictions
submision_lgb1['value'].to_csv('submision_lgb1.csv',index=False)


# In[ ]:


preds=pd.DataFrame(preds) 
preds.to_csv('new_preds.csv',index=False)


# In[ ]:


temp2=pd.DataFrame(preds)
result = temp2
result1 =result
result1['value']=0
numlist = np.array(result1)
gsl=np.argmax(numlist,axis=1)
#print(gsl)
i=0
k=len(gsl)
for i in range(k):
    result['value'][i]=gsl[i]+1


# In[ ]:


result1[['value']].to_csv('submit_B.csv',index=False)




import torch
import torch.nn as nn
import numpy as np

class SEBlock(nn.Module):
    def __init__(self, channel, reduction=16):
        super(SEBlock, self).__init__()
        self.avg_pool = nn.AdaptiveAvgPool2d(1)
        self.fc = nn.Sequential(
            nn.Linear(channel, channel // reduction, bias=False),
            nn.ReLU(inplace=True),
            nn.Linear(channel // reduction, channel, bias=False),
            nn.Sigmoid()
        )

    def forward(self, x):
        b, c, _, _ = x.size()
        y = self.avg_pool(x).view(b, c)
        y = self.fc(y).view(b, c, 1, 1)
        return x * y.expand_as(x)

class SpatialAttention(nn.Module):
    def __init__(self, kernel_size=7):
        super(SpatialAttentionBlock, self).__init__()
        self.conv1 = nn.Conv2d(2, 1, kernel_size=kernel_size, padding=kernel_size // 2, bias=False)
        self.sigmoid = nn.Sigmoid()

    def forward(self, x):
        # Compute the spatial attention scores
        avg_out = torch.mean(x, dim=1, keepdim=True)  # Shape: [batch_size, 1, height, width]
        max_out, _ = torch.max(x, dim=1, keepdim=True)  # Shape: [batch_size, 1, height, width]
        x = torch.cat([avg_out, max_out], dim=1)  # Shape: [batch_size, 2, height, width]
        x = self.conv1(x)  # Shape should remain: [batch_size, 1, height, width]
        return self.sigmoid(x)  # Applying sigmoid to scale values between 0 and 1

class ChannelAttention(nn.Module):
    def __init__(self, num_channels, reduction_ratio=16):
        super(ChannelAttention, self).__init__()
        self.avg_pool = nn.AdaptiveAvgPool2d(1)  # Global Average Pooling
        self.fc = nn.Sequential(
            nn.Linear(num_channels, num_channels // reduction_ratio, bias=False),
            nn.ReLU(inplace=True),
            nn.Linear(num_channels // reduction_ratio, num_channels, bias=False),
            nn.Sigmoid()
        )

    def forward(self, x):
        b, c, _, _ = x.size()
        y = self.avg_pool(x).view(b, c)
        y = self.fc(y).view(b, c, 1, 1)
        return x * y.expand_as(x)


class SpatialAttentionBlock(nn.Module):
    def __init__(self, kernel_size=7):
        super(SpatialAttentionBlock, self).__init__()
        assert kernel_size in (3, 7), 'kernel size must be 3 or 7'
        padding = 3 if kernel_size == 7 else 1
        self.conv1 = nn.Conv2d(2, 1, kernel_size, padding=padding, bias=False)
        self.sigmoid = nn.Sigmoid()

    def forward(self, x):
        avg_out = torch.mean(x, dim=1, keepdim=True)
        max_out, _ = torch.max(x, dim=1, keepdim=True)
        x = torch.cat([avg_out, max_out], dim=1)
        x = self.conv1(x)
        return x * self.sigmoid(x)



class ResBlock(nn.Module):
    def __init__(self, in_channels, out_channels, stride=1, kernel_size=3, dropout_rate=0.1):
        super(ResBlock, self).__init__()
        #self.se = SEBlock(out_channels)
        self.conv1 = nn.Conv2d(in_channels, out_channels, kernel_size=kernel_size, stride=stride, padding=kernel_size//2, bias=False)
        self.bn1 = nn.BatchNorm2d(out_channels)
        self.relu = nn.ReLU(inplace=True)
        self.dropout = nn.Dropout(dropout_rate)
        self.conv2 = nn.Conv2d(out_channels, out_channels, kernel_size=kernel_size, stride=1, padding=kernel_size//2, bias=False)
        self.bn2 = nn.BatchNorm2d(out_channels)
        self.spatial_attention = SpatialAttentionBlock()  # Add spatial attention block
        self.channel_attention = ChannelAttention(self.channel_list[0]) 
        if self.channel_list[0] != self.channel_list[1] or stride != 1:
            self.downsample = nn.Sequential(
                nn.Conv2d(self.channel_list[0], self.channel_list[1], kernel_size=1, stride=stride, bias=False),
                nn.BatchNorm2d(self.channel_list[0])
            )
        else:
            self.downsample = None

    def forward(self, x):
        identity = x
        out = self.conv1(x)
        out = self.bn1(out)
        out = self.relu(out)
        out = self.dropout(out)
        out = self.conv2(out)
        out = self.bn2(out)
        out = self.channel_attention(out)  # Apply SEBlock after the last BN
        # Apply spatial attention
        attention = self.spatial_attention(out)  # Generate attention map
        out = out * attention  # Apply attention map
        
        
        if self.downsample is not None:
            identity = self.downsample(x)

        out += identity
        out = self.relu(out)
        return out

class Neural_receiver(nn.Module):
    def __init__(self, subcarriers, timesymbols, streams, num_bits_per_symbol, num_blocks, base_channels, scene, dropout_rate=0.1):
        super(Neural_receiver, self).__init__()
        self.subcarriers = subcarriers
        self.timesymbols = timesymbols
        self.streams = streams
        self.num_blocks = num_blocks
        self.base_channels = base_channels
        self.num_bits_per_symbol = num_bits_per_symbol

        if scene == 'scene1':
            self.register_buffer('W', torch.full((1, 1, timesymbols, subcarriers, 2), np.sqrt(0.9)))
            self.register_buffer('V', torch.full((1, 1, timesymbols, subcarriers, 2), np.sqrt(0.1)))
        elif scene == 'scene2':
            self.register_buffer('W', torch.full((1, 1, timesymbols, subcarriers, 2), np.sqrt(0.6)))  # 修改数据权重以反映幅度比
            self.register_buffer('V', torch.zeros((1, 1, timesymbols, subcarriers, 2)))  # 初始化导频权重为0
            specific_t_V = {1: [0, 4, 8], 2: [1, 5, 9], 3: [2, 6, 10], 4: [3, 7, 11]}
            # 更新导频权重以反映根据功率比调整后的幅度比
            for l in range(streams):
                for t in specific_t_V[l + 1]:
                    self.V[0, 0, t, :, :] = np.sqrt(1.6)  # 使用功率比的平方根作为幅度比

        self.init_conv = nn.Conv2d(4 * self.streams, self.base_channels, kernel_size=3, padding=1)

        self.blocks = nn.Sequential()
        for block_id in range(self.num_blocks):
            in_channels = self.base_channels if block_id == 0 else layer_channels
            layer_channels = self.base_channels + (block_id // 10) * 32
            self.blocks.add_module(
                name=f'resblock_{block_id}',
                module=ResBlock(in_channels, layer_channels, dropout_rate=dropout_rate)
            )

        final_channels = self.base_channels + ((self.num_blocks - 1) // 10) * 32
        self.final_conv = nn.Conv2d(final_channels, streams * num_bits_per_symbol, kernel_size=3, padding=1)

    def forward(self, y, template_pilot):        
        y = y.float()
        template_pilot=template_pilot.float()
        # 假设 rx_signal 是您从文件加载的接收信号张量
        '''
        mean_real = torch.mean(y[..., 0], dim=(0, 1, 2), keepdim=True)
        std_real = torch.std(y[..., 0], dim=(0, 1, 2), keepdim=True)
        mean_imag = torch.mean(y[..., 1], dim=(0, 1, 2), keepdim=True)
        std_imag = torch.std(y[..., 1], dim=(0, 1, 2), keepdim=True)

        # 标准化
        y[..., 0] = (y[..., 0] - mean_real) / (std_real + 1e-6)
        y[..., 1] = (y[..., 1] - mean_imag) / (std_imag + 1e-6)
    '''

        #y = self.data_norm(y)  # Apply normalization
        y_weighted = y * self.W
        pilot_weighted = template_pilot * self.V
        
        batch_size = y.shape[0]
        y = y_weighted.permute(0, 2, 3, 1, 4)
        y = torch.reshape(y, (batch_size, self.timesymbols, self.subcarriers, self.streams * 2))
        template_pilot = pilot_weighted.permute(0, 2, 3,1, 4)
        template_pilot = torch.reshape(template_pilot, (batch_size, self.timesymbols, self.subcarriers, self.streams * 2))

        z = torch.cat([y, template_pilot], dim=-1)
        z = z.permute(0, 3, 1, 2)
        z = self.init_conv(z)
        z = self.blocks(z)
        z = self.final_conv(z)
        z = z.permute(0, 2, 3, 1)
        z = torch.reshape(z, (batch_size, self.timesymbols, self.subcarriers, self.streams, self.num_bits_per_symbol))
        z = z.permute(0, 3, 1, 2, 4)
        return z



#!/usr/bin/env python
# coding: utf-8

# In[ ]:


'''
使用附加信息预测时间序列
在实际的预测问题中，我们经常可以获得除原始时间序列值之外的其他信息。AutoGluon 支持两种类型的此类附加信息：静态特征和随时间变化的协变量。

静态特征
静态特征是时间序列中与时间无关的属性（元数据）。这些可能包括以下信息：

记录时间序列的位置（国家、州、城市）

产品的固定属性（品牌名称、颜色、尺寸、重量）

商店 ID 或产品 ID

例如，提供这些信息可能有助于预测模型为位于同一城市的商店生成类似的需求预测。

在 AutoGluon 中，静态特征存储为TimeSeriesDataFrame对象的属性。作为示例，我们来看看 M4 Daily 数据集。
'''


# In[ ]:


import pandas as pd
from autogluon.timeseries import TimeSeriesDataFrame, TimeSeriesPredictor


# In[ ]:


df = pd.read_csv("https://autogluon.s3.amazonaws.com/datasets/timeseries/m4_daily_subset/train.csv")
df.head()


# In[ ]:


static_features_df = pd.read_csv("https://autogluon.s3.amazonaws.com/datasets/timeseries/m4_daily_subset/metadata.csv")
static_features_df.head()


# In[ ]:


train_data = TimeSeriesDataFrame.from_data_frame(
    df,
    id_column="item_id",
    timestamp_column="timestamp",
    static_features_df=static_features_df,
)
train_data.head()


# In[ ]:


train_data.static_features.head()


# In[ ]:


predictor = TimeSeriesPredictor(prediction_length=14).fit(train_data)


# In[ ]:


#默认情况下，此列将被解释为连续数字。我们可以通过将 dtype 更改为 来强制 AutoGluon 将其解释为分类特征category。
train_data.static_features["store_id"] = list(range(len(train_data.item_ids)))
train_data.static_features["store_id"] = train_data.static_features["store_id"].astype("category")


# In[ ]:


df_irregular = TimeSeriesDataFrame(
    pd.DataFrame(
        {
            "item_id": [0, 0, 0, 1, 1],
            "timestamp": ["2022-01-01", "2022-01-02", "2022-01-04", "2022-01-01", "2022-01-04"],
            "target": [1, 2, 3, 4, 5],
        }
    )
)
df_irregular


# In[ ]:


#处理不规则数据和缺失值
#在这种情况下，您可以在使用参数创建预测器时指定所需的频率freq。
predictor = TimeSeriesPredictor(..., freq="D").fit(df_irregular)
#AutoGluon 会自动将不规则数据转换为每日频率并处理缺失值。
df_regular = df_irregular.convert_frequency(freq="D")
df_regular
#或者，我们可以使用TimeSeriesDataFrame.fill_missing_values()以适当的策略手动填充 NaN 。默认情况下，缺失值使用前向 + 后向填充的组合进行填充。
df_filled = df_regular.fill_missing_values()
df_filled
#在某些应用（例如需求预测）中，缺失值可能对应于零需求。在这种情况下，常量填充更为合适。
df_filled = df_regular.fill_missing_values(method="constant", value=0.0)
df_filled


# In[ ]:


#如何评估预测准确率？
prediction_length = 48
data = TimeSeriesDataFrame.from_path("https://autogluon.s3.amazonaws.com/datasets/timeseries/m4_hourly_subset/train.csv")
train_data, test_data = data.train_test_split(prediction_length)


# In[ ]:


import matplotlib.pyplot as plt
import numpy as np

item_id = "H1"
fig, (ax1, ax2) = plt.subplots(nrows=2, figsize=[10, 4], sharex=True)
train_ts = train_data.loc[item_id]
test_ts = test_data.loc[item_id]
ax1.set_title("Train data (past time series values)")
ax1.plot(train_ts)
ax2.set_title("Test data (past + future time series values)")
ax2.plot(test_ts)
for ax in (ax1, ax2):
    ax.fill_between(np.array([train_ts.index[-1], test_ts.index[-1]]), test_ts.min(), test_ts.max(), color="C1", alpha=0.3, label="Forecast horizon")
plt.legend()
plt.show()


# In[ ]:


predictor = TimeSeriesPredictor(...)
predictor.fit(train_data, presets="medium_quality")

'''
fast_training
medium_quality
high_quality
best_quality
'''
'''
ETS
AutoARIMA
Theta
SeasonalNaive
'''

'''
DeepAR

PatchTST

DLinear

TemporalFusionTransformer
'''




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import pandas as pd
import numpy as np
from matplotlib import pyplot as plt
import seaborn as sns
from sklearn import cluster as clu
from sklearn import pipeline as pip
from sklearn import metrics as met
from sklearn import decomposition as dec
from sklearn import preprocessing as pre
from sklearn import manifold as man
from sklearn import compose as cmp
import plotly.graph_objects as go
from sklearn import impute as imp
import sklearn.neighbors as nn


# In[ ]:


df = pd.read_csv("https://raw.githubusercontent.com/ageron/handson-ml/master/datasets/housing/housing.csv")
df.head(5)


# In[ ]:


df.describe(include="all").T


# In[ ]:


X_train = df.drop(columns=["median_house_value"],axis=1)


# In[ ]:


fig = go.Figure(data=go.Scattergeo(
        lon = df['longitude'],
        lat = df['latitude'],
        text = df['median_house_value'],
        mode = 'markers',
        marker_color = df['median_house_value'],      
        ))
fig.show()


# In[ ]:


import plotly.graph_objects as go

import pandas as pd

fig = go.Figure(data=go.Scattergeo(
        locationmode = 'USA-states',
        lon = df['longitude'],
        lat = df['latitude'],
        text = df['median_house_value'],
        mode = 'markers',
        marker = dict(
            #size = 8,
            opacity = 0.8,
            reversescale = True,
            autocolorscale = False,
            #symbol = 'square',
            line = dict(
                width=1,
                color='rgba(102, 102, 102)'
            ),
            colorscale = 'RdBu',
            cmin = 0,
            color = df['median_house_value'],
            cmax = df['median_house_value'].max(),
            colorbar_title="median_house_value"
        )))

fig.update_layout(
        #title = 'median_house_value)',
        geo = dict(
            scope='usa',
            projection_type='albers usa',
            showland = True,
            landcolor = "rgb(250, 250, 250)",
            subunitcolor = "rgb(217, 217, 217)",
            countrycolor = "rgb(217, 217, 217)",
            countrywidth = 0.5,
            subunitwidth = 0.5
        ),
    )
fig.show()


# # Kmeans

# In[ ]:


num_list = list(X_train.select_dtypes("number").columns)
cat_list = list(X_train.select_dtypes(exclude="number").columns)

pipe_num = pip.Pipeline([
                        ("imp",imp.SimpleImputer(strategy="mean")),
                        ("scl",pre.StandardScaler()) 
                        ]) 

pipe_cat = pip.Pipeline([
                        ("ohe",pre.OneHotEncoder())
                        ]) 
preprocessor = cmp.ColumnTransformer([
                                      ("numimp",pipe_num,num_list),
                                      ("cat",pipe_cat,cat_list)],
                                      remainder="passthrough")



# In[ ]:


x_values = []
y_values = []
for i in range(1,40):
    pipe = pip.Pipeline([
                        ("pre",preprocessor),
                        ("clu",clu.KMeans(n_clusters=i,random_state=42))
                        ])
    pipe.fit(X_train)
    x_values.append(i)
    y_values.append(pipe.named_steps["clu"].inertia_)
plt.plot(x_values,y_values)
plt.show()


# In[ ]:


# 10 seems okay


# In[ ]:


pipe = pip.Pipeline([
                    ("pre",preprocessor),
                    ("clu",clu.KMeans(n_clusters=10,random_state=42))
                    ])
pipe.fit(X_train)


# In[ ]:


import plotly.graph_objects as go

import pandas as pd

fig = go.Figure(data=go.Scattergeo(
        locationmode = 'USA-states',
        lon = X_train['longitude'],
        lat = X_train['latitude'],
        text = pipe[1].labels_,
        mode = 'markers',
        marker = dict(
            #size = 8,
            #opacity = 0.5,
            reversescale = True,
            #autocolorscale = False,
            #symbol = 'square',
            line = dict(
                width=1,
                color='rgba(102, 102, 102)'
            ),
            colorscale = 'RdBu',
            cmin = 0,
            color = pipe[1].labels_,
            cmax = pipe[1].labels_.max(),
            colorbar_title="median_house_value"
        )))

fig.update_layout(
        title = 'Cluesters',
        geo = dict(
            scope='usa',
            projection_type='albers usa',
            showland = True,
            landcolor = "rgb(250, 250, 250)",
            subunitcolor = "rgb(217, 217, 217)",
            countrycolor = "rgb(217, 217, 217)",
            countrywidth = 0.5,
            subunitwidth = 0.5
        ),
    )
fig.show()


# # DBscan

# In[ ]:


num_list = list(X_train.select_dtypes("number").columns)
cat_list = list(X_train.select_dtypes(exclude="number").columns)

pipe_num = pip.Pipeline([
                        ("imp",imp.SimpleImputer(strategy="mean")),
                        ("scl",pre.StandardScaler()) 
                        ]) 

pipe_cat = pip.Pipeline([
                        ("ohe",pre.OneHotEncoder())
                        ]) 
preprocessor = cmp.ColumnTransformer([
                                      ("numimp",pipe_num,num_list),
                                      ("cat",pipe_cat,cat_list)],
                                      remainder="passthrough")

pipe = pip.Pipeline([
                    ("pre",preprocessor),
                    #("nn",nn.NearestNeighbors(n_neighbors=10,algorithm="auto"))
                    ])
pipe.fit(X_train)


# In[ ]:


X_train_transformed = pipe.fit_transform(X_train)


# In[ ]:


nn_obj = nn.NearestNeighbors(n_neighbors=10,algorithm="auto")
nn_obj.fit(X_train_transformed)
output = nn_obj.kneighbors(X_train_transformed, return_distance=True)


# In[ ]:


X_train_transformed.shape


# In[ ]:


print("indices",output[0].shape)
print("distances",output[1].shape)


# In[ ]:


mean_distances = list(pd.DataFrame(output[0]).mean(axis=1).sort_values().values)
mean_indexes = list(pd.DataFrame(output[1]).index)
i = 0
for i in range(5):
    print(mean_indexes[i],mean_distances[i])


# In[ ]:


plt.plot(mean_indexes,mean_distances)
plt.show()


# In[ ]:


plt.plot(mean_indexes,mean_distances)
plt.ylim(0, 3)
plt.show()


# In[ ]:


# 1 seems to be fine


# In[ ]:


num_list = list(X_train.select_dtypes("number").columns)
cat_list = list(X_train.select_dtypes(exclude="number").columns)

pipe_num = pip.Pipeline([
                        ("imp",imp.SimpleImputer(strategy="mean")),
                        ("scl",pre.StandardScaler()) 
                        ]) 

pipe_cat = pip.Pipeline([
                        ("ohe",pre.OneHotEncoder())
                        ]) 
preprocessor = cmp.ColumnTransformer([
                                      ("numimp",pipe_num,num_list),
                                      ("cat",pipe_cat,cat_list)],
                                      remainder="passthrough")

pipe = pip.Pipeline([
                    ("pre",preprocessor),
                    ("clu",clu.DBSCAN(eps=1,min_samples=10))
                    ])
pipe.fit(X_train)


# In[ ]:


import plotly.graph_objects as go

import pandas as pd

fig = go.Figure(data=go.Scattergeo(
        locationmode = 'USA-states',
        lon = X_train['longitude'],
        lat = X_train['latitude'],
        text = pipe[1].labels_,
        mode = 'markers',
        marker = dict(
            #size = 8,
            #opacity = 0.5,
            reversescale = True,
            #autocolorscale = False,
            #symbol = 'square',
            line = dict(
                width=1,
                color='rgba(102, 102, 102)'
            ),
            colorscale = 'RdBu',
            cmin = 0,
            color = pipe[1].labels_,
            cmax = pipe[1].labels_.max(),
            colorbar_title=""
        )))

fig.update_layout(
        #title = 'Cluesters',
        geo = dict(
            scope='usa',
            projection_type='albers usa',
            showland = True,
            landcolor = "rgb(250, 250, 250)",
            subunitcolor = "rgb(217, 217, 217)",
            countrycolor = "rgb(217, 217, 217)",
            countrywidth = 0.5,
            subunitwidth = 0.5
        ),
    )
fig.show()




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from pycaret.datasets import get_data
data = get_data('./datasets/anomaly')
# data = get_data('anomaly')


# In[ ]:


from pycaret.anomaly import AnomalyExperiment
s = AnomalyExperiment()
s.setup(data, session_id = 0)
# 另一种加载方式
# from pycaret.anomaly import *
# s = setup(data, session_id = 0)


# In[ ]:


iforest = s.create_model('iforest')
print(iforest)


# In[ ]:


IForest(behaviour='new', bootstrap=False, contamination=0.05,
    max_features=1.0, max_samples='auto', n_estimators=100, n_jobs=-1,
    random_state=0, verbose=0)


# In[ ]:


s.models()


# In[ ]:


result = s.assign_model(iforest)
result.head()


# In[ ]:


predictions = s.predict_model(iforest, data = data)
predictions.head()


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[ ]:


from sklearn.decomposition import PCA #主成分分析法
import os
import math
import time
import random
import datetime
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from sklearn.model_selection import KFold
from sklearn.preprocessing import OneHotEncoder
from sklearn.metrics import roc_auc_score
import json
import warnings
from sklearn import preprocessing
import lightgbm as lgb  
import pickle  
from sklearn.model_selection import train_test_split  
from sklearn.metrics import roc_auc_score, roc_curve,mean_squared_error
from sklearn.model_selection import KFold,GridSearchCV
from sklearn import metrics
from joblib import Parallel, delayed
pd.set_option('display.max_columns',None)
warnings.filterwarnings("ignore")
warnings.warn("once")
from scipy.interpolate import interp1d
from scipy.special import erf, erfinv
from sklearn.preprocessing import QuantileTransformer,PowerTransformer
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.utils.validation import FLOAT_DTYPES, check_array, check_is_fitted
 


# In[ ]:


def handleOutlier(df, cols=None, detect="value", method="median"):
    if not cols:
        # 如果用户没有输入，则填充全部
        cols = df.columns.tolist()
    if detect == "value":
        if method == "median":
            for name in cols:
                if df[name].dtype != 'object':
                    # 中位数插补，适用于偏态分布或者有离群点的分布
                    med = df[name].median()
                    mean = df[name].mean()
                    std = df[name].std()

                    df[name] = df[name].map(lambda x: med if abs(x - mean) > 3 * std else x)
        elif method == "mode":
            for name in cols:
                # 众数插补，可用于str，int，float
                if df[name].dtype != 'object':
                    mode = df[name].mode()[0]
                    mean = df[name].mean()
                    std = df[name].std()

                    df[name] = df[name].map(lambda x: mode if abs(x - mean) > 3 * std else x)
        else:
            return IOError
    elif detect == "frequency":
        # 根据数据出现频率检测异常值
        # 待添加
        return df
    return df


# In[ ]:


# 导入数据  
print("Loading Data ... ")  
new_train_data = pd.read_csv('/home/bwfy/桌面/比赛赛集/产品评分数据集/Train.csv')
new_test_data = pd.read_csv('/home/bwfy/桌面/比赛赛集/产品评分数据集/Test.csv')
submi=pd.read_csv('/home/bwfy/桌面/比赛赛集/submision_lgb1.csv')


# In[ ]:


new_train_data=handleOutlier(new_train_data,detect="value", method="median")
new_test_data=handleOutlier(new_test_data,detect="value", method="median")


# In[ ]:


#合并字段
data_all = pd.concat([new_train_data.assign(is_train = 1),new_test_data.assign(is_train = 0)],ignore_index=True) #合并train和test，并且用is_train进行标记
train = data_all['is_train'] == 1##提前进行标记
test  = data_all['is_train'] == 0
print('数据全集量是',len(data_all))
train_count = len(data_all[train])
print('训练集样本量是',train_count)
test_count = len(data_all[test])
print('测试集样本量是',test_count)
print('样本比例为：', train_count/test_count)

#data_all=handleOutlier(data_all,detect="value", method="median")


# In[ ]:


data_all_B = data_all.copy()


# In[ ]:


data_all_B.describe()


# In[ ]:


data_all_B['value'] =   data_all['value']
data_all_B['is_train'] = data_all['is_train']


# In[ ]:


data_all['KPI_A_5']= data_all['KPI_A_5'].astype('category')
data_all['KPI_A_7']= data_all['KPI_A_7'].astype('category')


# In[ ]:


data_all_B.drop(["KPI_A_10",'KPI_A_11','KPI_A_5','KPI_A_7','KPI_A_12',],axis=1,inplace=True)


# In[ ]:


train_data_1 = data_all_B[data_all_B['is_train']== 1]
test_data_1  = data_all_B[data_all_B['is_train']== 0]


# In[ ]:


train_data_1.to_csv('T1_gs_train.csv',index=False)
test_data_1.to_csv('T1_gs_test.csv',index=False)


# In[ ]:


train_data_1 = train_data_1.drop(['is_train'],axis=1)
test_data_1 = test_data_1.drop(['is_train','value'],axis=1)


# In[ ]:


print(submi)


# In[ ]:


test_data_1=test_data_1.reset_index()


# In[ ]:


test_data_1 ['value'] = submi['value']


# In[ ]:


print(test_data_1)


# In[ ]:


train_data_dall = pd.concat([train_data_1.assign(is_train = 1),test_data_1.assign(is_train = 0)],ignore_index=True) #合并train和test，并且用is_train进行标记


# In[ ]:


train_data_dall.to_csv('train_data_dall.csv',index=False)


# In[ ]:


train_data_dall = train_data_dall.drop(['is_train','index'],axis=1)


# In[ ]:


test_data_1 = test_data_1.drop(['value'],axis=1)


# In[ ]:


# 选择相关性大于0.05的作为候选特征参与训练，并加入我们认为比较重要的特征，总共66个特征参与训练
features = (train_data_dall.corr()['value'][abs(train_data_dall.corr()['value'])!='K']).index
features = features.values.tolist()
features.remove('value')
len(features)


# In[ ]:


# 生成数据和标签
target = train_data_dall['value']
train_selected = train_data_dall[features]
test = test_data_1[features]
feature_importance_df = pd.DataFrame()
oof = np.zeros(len(train_data_dall))
predictions = np.zeros(len(test_data_1))


# In[ ]:


print (15**2)


# In[ ]:


params ={#'N_estimator':10000, #3000
         'num_leaves': 197,#最大值不超过 2^'max_depth'# 2**7  800
         'min_data_in_leaf': 40,
         'objective': 'regression',#regression_l1
         'max_depth': 15,#19
         'learning_rate': 0.03,#0.03
         'boosting': 'gbdt',
         'bagging_freq': 0,
         'bagging_fraction': 0.8,   # 每次迭代时用的数据比例0.8
         'feature_fraction': 0.9,# 每次迭代中随机选择80％的参数来建树0.8201
         'bagging_seed': 11,
          'min_data_in_leaf':1,
            'max_bin': 298,
        'lambda_l1': 0,
          'lambda_l2': 0,
    #   'reg_alpha': 1.728910519108444,
       #  'reg_lambda': 4.9847051755586085,
         'random_state': 42,
         'metric': {'mae'},
         'verbosity': -1,
         'subsample': 0.81,
         'min_gain_to_split': 0.01077313523861969,
         'min_child_weight': 19.428902804238373,
         'num_threads': 8,
         'seed':2021}
          #zero_as_missing=true 会将 0 也当作缺失值处理。
kfolds = KFold(n_splits=5,shuffle=True,random_state=15)#15

predictions = np.zeros(len(test))

#categorical_feature=
#cat=['KPI_A_5','KPI_A_7','KPI_A_12','KPI_A_9'] 

for fold_n,(trn_index,val_index) in enumerate(kfolds.split(train_selected,target)):
    print("fold_n {}".format(fold_n))
    trn_data = lgb.Dataset(train_selected.iloc[trn_index],label=target.iloc[trn_index])
    val_data = lgb.Dataset(train_selected.iloc[val_index],label=target.iloc[val_index])
    num_round=100000
    clf = lgb.train(params, trn_data, num_round, valid_sets = [trn_data, val_data],   
                    verbose_eval=1000, early_stopping_rounds = 2000)#categorical_feature=cat
    oof[val_index] = clf.predict(train_selected.iloc[val_index], num_iteration=clf.best_iteration)
    predictions += clf.predict(test,num_iteration=clf.best_iteration)/5
    fold_importance_df = pd.DataFrame()
    fold_importance_df["feature"] = features
    fold_importance_df["importance"] = clf.feature_importance()
    fold_importance_df["fold"] = fold_n + 1
    feature_importance_df = pd.concat([feature_importance_df, fold_importance_df], axis=0)
    print("CV score: {:<8.5f}".format(mean_squared_error(target, oof)**0.5))


# In[ ]:


cols = (feature_importance_df[["feature", "importance"]]
        .groupby("feature")
        .mean()
        .sort_values(by="importance", ascending=False)[:1000].index)
best_features = feature_importance_df.loc[feature_importance_df.feature.isin(cols)]

plt.figure(figsize=(14,26))
sns.barplot(x="importance", y="feature", data=best_features.sort_values(by="importance",ascending=False))
plt.title('LightGBM Features (averaged over folds)')
plt.tight_layout()


# In[ ]:


### best_features.to_csv('best_features.csv',index=False)


# In[ ]:


# 计算结果
submision_lgb1=test
submision_lgb1['value']=predictions
submision_lgb1['value'].to_csv('submision_lgb2.csv',index=False)




import torch
import torch.nn as nn

class ResBlock(nn.Module):
    def __init__(self, channel_list, H, W):
        super(ResBlock, self).__init__()
        # 调整通道数为更高
        self.channel_list = [max(ch, 256) for ch in channel_list]
        self._conv_1 = nn.Conv2d(self.channel_list[2], self.channel_list[0], kernel_size=3, padding=1)
        self._bn_1 = nn.BatchNorm2d(self.channel_list[0])
        self._conv_2 = nn.Conv2d(self.channel_list[0], self.channel_list[1], kernel_size=3, padding=1)
        self._bn_2 = nn.BatchNorm2d(self.channel_list[1])
        self._relu = nn.ReLU()
        self._dropout = nn.Dropout(0.5)  # 调整Dropout比率

    def forward(self, inputs):
        x_ini = inputs
        x = self._bn_1(self._conv_1(x_ini))
        x = self._relu(x)
        x = self._dropout(x)
        x = self._bn_2(self._conv_2(x))
        x = self._relu(x)
        x = self._dropout(x)
        x_ini = x_ini + x
        return x_ini

class Neural_receiver(nn.Module):
    def __init__(self, subcarriers, timesymbols, streams, num_bits_per_symbol, num_blocks=20, channel_list=[256, 256, 256]):
        super(Neural_receiver, self).__init__()
        self.subcarriers = subcarriers
        self.timesymbols = timesymbols
        self.streams = streams
        # 增加残差块的数量
        self.num_blocks = num_blocks
        self.channel_list = channel_list
        self.num_bits_per_symbol = num_bits_per_symbol

        self.blocks = nn.Sequential()
        for block_id in range(self.num_blocks):
            block = ResBlock(channel_list=self.channel_list, H=self.timesymbols, W=self.subcarriers)
            self.blocks.add_module(name='block_{}'.format(block_id), module=block)
        self._conv_1 = nn.Conv2d(4 * self.streams, self.channel_list[2], kernel_size=3, padding=1)
        self._conv_2 = nn.Conv2d(self.channel_list[1], self.streams * self.num_bits_per_symbol, kernel_size=3, padding=1)

    def forward(self, y, template_pilot):
        batch_size = y.shape[0]
        y = y.permute(0, 2, 3, 1, 4)
        y = torch.reshape(y, (batch_size, self.timesymbols, self.subcarriers, self.streams * 2))
        template_pilot = template_pilot.permute(0, 2, 3, 1, 4)
        template_pilot = torch.reshape(template_pilot, (batch_size, self.timesymbols, self.subcarriers, self.streams * 2))

        z = torch.cat([y, template_pilot], dim=-1)
        z = z.permute(0, 3, 1, 2)
        z = self._conv_1(z)
        z = self.blocks(z)
        z = self._conv_2(z)
        z = z.permute(0, 2, 3, 1)
        z = torch.reshape(z, (batch_size, self.timesymbols, self.subcarriers, self.streams, self.num_bits_per_symbol))
        z = z.permute(0, 3, 1, 2, 4)
        return z



#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import os
import math
import time
import random
import datetime
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
import tensorflow as tf
import tensorflow.keras as K
from tensorflow.keras import Sequential, utils, regularizers, Model, Input
from tensorflow.keras.layers import Flatten, Dense, Conv1D, MaxPool1D, Dropout, AvgPool1D
from imblearn.over_sampling import SMOTE
from sklearn.model_selection import KFold
from sklearn.preprocessing import OneHotEncoder

import pickle
from scipy.sparse import csr_matrix, hstack
from collections import Counter
from scipy.stats import entropy
from statistics import mean
from keras import regularizers
import warnings
warnings.filterwarnings("ignore")
from sklearn.model_selection import train_test_split, RandomizedSearchCV
from sklearn.preprocessing import OneHotEncoder, LabelBinarizer, Normalizer, LabelEncoder, StandardScaler
from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer, HashingVectorizer
import os
from sklearn.neighbors import KNeighborsClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier
from xgboost import XGBClassifier
import xgboost as xgb
from lightgbm import LGBMClassifier
import lightgbm as lgb

import tensorflow as tf
from tensorflow.keras.layers import Input, Dense, Conv2D, MaxPool2D, Activation, Dropout, Flatten, PReLU, BatchNormalization
from tensorflow.keras.applications import VGG16
from tensorflow.keras.models import Model, Sequential
from tensorflow.keras.callbacks import Callback, ModelCheckpoint, TensorBoard, EarlyStopping, ReduceLROnPlateau
from tensorflow.keras.preprocessing.image import ImageDataGenerator
from tensorflow.keras.utils import plot_model
from tensorflow.keras.optimizers import Adagrad, Adam, Adadelta, Adamax
from keras.utils import np_utils
from sklearn.calibration import CalibratedClassifierCV
from sklearn.metrics import log_loss
from prettytable import PrettyTable
train = pd.read_csv('datasets/@huangzx23#ba5d051838cf583c27a68007e887d01c/train.csv')
test = pd.read_csv('datasets/@huangzx23#ba5d051838cf583c27a68007e887d01c/test.csv')
y_train = train['label']
x_train = train.drop(['label'], axis=1)
X_test = test
print(x_train.shape, y_train.shape)
print(X_test.shape)


# In[ ]:


import numpy as np
from joblib import Parallel, delayed
from scipy.interpolate import interp1d
from scipy.special import erf, erfinv
from sklearn.preprocessing import QuantileTransformer,PowerTransformer
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.utils.validation import FLOAT_DTYPES, check_array, check_is_fitted
class GaussRankScaler(BaseEstimator, TransformerMixin):
    def __init__(self, epsilon=1e-4, copy=True, n_jobs=None, interp_kind='linear', interp_copy=False):
        self.epsilon     = epsilon
        self.copy        = copy
        self.interp_kind = interp_kind
        self.interp_copy = interp_copy
        self.fill_value  = 'extrapolate'
        self.n_jobs      = n_jobs
 
    def fit(self, X, y=None):
       
        X = check_array(X, copy=self.copy, estimator=self, dtype=FLOAT_DTYPES, force_all_finite=True)
 
        self.interp_func_ = Parallel(n_jobs=self.n_jobs)(delayed(self._fit)(x) for x in X.T)
        return self
 
    def _fit(self, x):
        x = self.drop_duplicates(x)
        rank = np.argsort(np.argsort(x))
        bound = 1.0 - self.epsilon
        factor = np.max(rank) / 2.0 * bound
        scaled_rank = np.clip(rank / factor - bound, -bound, bound)
        return interp1d(
            x, scaled_rank, kind=self.interp_kind, copy=self.interp_copy, fill_value=self.fill_value)
 
    def transform(self, X, copy=None):
       
        check_is_fitted(self, 'interp_func_')
 
        copy = copy if copy is not None else self.copy
        X = check_array(X, copy=copy, estimator=self, dtype=FLOAT_DTYPES, force_all_finite=True)
 
        X = np.array(Parallel(n_jobs=self.n_jobs)(delayed(self._transform)(i, x) for i, x in enumerate(X.T))).T
        return X
 
    def _transform(self, i, x):
        return erfinv(self.interp_func_[i](x))
 
    def inverse_transform(self, X, copy=None):
        
        check_is_fitted(self, 'interp_func_')
 
        copy = copy if copy is not None else self.copy
        X = check_array(X, copy=copy, estimator=self, dtype=FLOAT_DTYPES, force_all_finite=True)
 
        X = np.array(Parallel(n_jobs=self.n_jobs)(delayed(self._inverse_transform)(i, x) for i, x in enumerate(X.T))).T
        return X
 
    def _inverse_transform(self, i, x):
        inv_interp_func = interp1d(self.interp_func_[i].y, self.interp_func_[i].x, kind=self.interp_kind,
                                   copy=self.interp_copy, fill_value=self.fill_value)
        return inv_interp_func(erf(x))
 
    @staticmethod
    def drop_duplicates(x):
        is_unique = np.zeros_like(x, dtype=bool)
        is_unique[np.unique(x, return_index=True)[1]] = True
        return x[is_unique]


# In[ ]:


#查看类别分布
train2 = pd.read_csv('submit_B.csv')

plt.hist(train['label'], orientation = 'vertical', histtype = 'bar', color = 'red')
plt.show() 

plt.hist(train2['label'], orientation = 'vertical', histtype = 'bar', color = 'red')
plt.show() 


# In[ ]:


# 使用 SMOTE 对数据进行上采样以解决类别不平衡问题
smote = SMOTE(random_state=2021, n_jobs=-1)
k_x_train, k_y_train = smote.fit_resample(x_train, y_train)  
print(f"after smote, k_x_train.shape: {k_x_train.shape}, k_y_train.shape: {k_y_train.shape}")

# 将训练集转换为适应 CNN 输入的 shape
#k_x_train = np.array(k_x_train).reshape(k_x_train.shape[0], k_x_train.shape[1], 1)
k_x_train = k_x_train
k_y_train = k_y_train


# In[ ]:


print(k_x_train)


# In[ ]:


plt.hist(k_y_train, orientation = 'vertical', histtype = 'bar', color = 'blue')
plt.show() 


# In[ ]:


feature_names = k_x_train.columns.values.tolist() 
scaler_linear    = GaussRankScaler(interp_kind='linear',) 
for c in feature_names:
    k_x_train[c+'_linear_grank'] = scaler_linear.fit_transform(k_x_train[c].values.reshape(-1,1))
    
gaussian_linear_feature_names = [c + '_linear_grank' for c in feature_names]


# In[ ]:


feature_names = X_test.columns.values.tolist() 
scaler_linear    = GaussRankScaler(interp_kind='linear',) 
for c in feature_names:
    X_test[c+'_linear_grank'] = scaler_linear.fit_transform(X_test[c].values.reshape(-1,1))  
gaussian_linear_feature_names = [c + '_linear_grank' for c in feature_names]


# In[ ]:


os.environ['PYTHONHASHSEED'] = '0'
tf.keras.backend.clear_session()


# In[ ]:


#---------------------------------------------------------------
# Defining CALLBACKS
#---------------------------------------------------------------
filepath = "model_save/weights-{epoch:02d}-{val_accuracy:.04f}.hdf5"
checkPoint = ModelCheckpoint(filepath=filepath, monitor='val_accuracy', 
                             verbose=1, save_best_only=True, mode='auto')

#logDir = 'logs'
#tensorBoard_callback = TensorBoard(log_dir=logDir, histogram_freq=1, write_graph=True)

earlyStop = EarlyStopping(monitor='val_loss', min_delta=0, patience=5, 
                          verbose=1, mode='auto', restore_best_weights=True)

reduceLR = ReduceLROnPlateau(monitor='val_loss', factor=0.1, 
                             patience=3, verbose=1, mode='auto')
callBacks = [reduceLR, earlyStop]


# In[ ]:


def noEvModel(input_dim, output_dim):
    model = Sequential()
    model.add(Input(shape=(input_dim,)))
    model.add(Dense(256, activation='tanh', kernel_initializer=tf.keras.initializers.GlorotUniform(seed=44)))
    model.add(PReLU(alpha_initializer="zeros"))
    model.add(BatchNormalization())
    model.add(Dropout(0.5))
    model.add(Dense(4, activation='softmax', kernel_initializer=tf.keras.initializers.GlorotUniform(seed=46)))
    model.compile(loss='categorical_crossentropy', optimizer='adamax', metrics=['accuracy'])
    return model


# In[ ]:


print(k_x_train.shape)


# In[ ]:


print(k_y_train.shape)


# In[ ]:


noEventModel = noEvModel(k_x_train.shape[1], 4)
noEventModel.summary()


# In[ ]:


def noEventsModel(shuffle):
    """
    Takes a list of Random Seeds, splits the data into Train and CV based on Seed, trains model and takes average of 
    predictions while testing  
    """
    model_list=[]
    loss_list=[]
    avg_cv_loss=0
    for i in range(len(shuffle)):
        print('--Iteration #', i)
        X_tr, X_cr, y_train, y_cv = train_test_split(k_x_train, k_y_train, stratify=k_y_train, test_size=0.15, random_state=shuffle[i])
        
        y_train=np_utils.to_categorical(y_train)
        y_cv=np_utils.to_categorical(y_cv)
        model=noEvModel(X_tr.shape[1], 4)
        model.fit(X_tr, y_train, batch_size=256, epochs=100, verbose=2, shuffle=True, validation_data=(X_cr, y_cv),callbacks=callBacks)
        model.save('saved_models/noEvents/nn '+str(i+1))
        pred=model.predict(X_cr)
        cv_loss=log_loss(y_cv, pred)
        print("Validation Log Loss of  Model in Current Run: ", cv_loss)
        model_list.append(model)
        loss_list.append(cv_loss)
    avg_cv_loss = mean(loss_list)
    print("Average CV Loss of 6 Runs :", avg_cv_loss)
    return(model_list)


# In[ ]:


shuffle = [7, 14, 21, 28, 35, 42]
nnNoEvModel = noEventsModel(shuffle)


# In[ ]:


X_train, X_cv, y_train, y_cv = train_test_split(k_x_train, k_y_train, test_size=0.2, random_state=0, stratify=k_y_train)


# In[ ]:


avg_pred=np.zeros((X_train.shape[0],4))
for i in range(len(nnNoEvModel)):
    train_pred=nnNoEvModel[i].predict(X_train)
    avg_pred+=train_pred
avg_pred/=len(nnNoEvModel)
print("Train Average Log-Loss: ",log_loss(y_train, avg_pred))


# In[ ]:


avg_pred=np.zeros((X_cv.shape[0],4))
for i in range(len(nnNoEvModel)):
    cv_pred = nnNoEvModel[i].predict(X_cv)
    avg_pred += cv_pred
avg_pred/=len(nnNoEvModel)
print("CV Average Log-Loss: ",log_loss(y_cv, avg_pred))


# In[ ]:


prednoEv=np.zeros((X_test.shape[0],4))
for i in range(len(nnNoEvModel)):
    te_pred = nnNoEvModel[i].predict(X_test)
    prednoEv += te_pred
prednoEv/=len(nnNoEvModel)


# In[ ]:


pred = noEventModel.predict(X_train)
llNNnoEVTr=log_loss(y_train, pred)
print("Training Log Loss of NN Model: ",llNNnoEVTr)
pred = noEventModel.predict(X_cv)
llNNnoEVCv=log_loss(y_cv, pred)
print("Validation Log Loss of NN Model: ",llNNnoEVCv)


# In[ ]:


prednoEv = noEventModel.predict(X_test)


# In[ ]:


y_test_pred = np.argmax(prednoEv, axis=1)


# In[ ]:


print(prednoEv)


# In[ ]:


print(y_test_pred)#[3 3 3 ... 3 1 3]

